/*****************************************************************************
 *
 * UTILS.C - Miscellaneous utility functions for NetSaint
 *
 * Copyright (c) 1999-2000 Ethan Galstad (netsaint@linuxbox.com)
 * Last Modified:   01-22-2000
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "../common/config.h"
#include "../common/common.h"

#ifdef HAVE_SYS_RESOURCE_H
#include <sys/resource.h>
#endif
 
#include "netsaint.h"

char            *my_strtok_buffer=NULL;
char            *original_my_strtok_buffer=NULL;


extern char	config_file[MAX_FILENAME_LENGTH];
extern char     hosts_config_file[MAX_FILENAME_LENGTH];
extern char	log_file[MAX_FILENAME_LENGTH];
extern char     status_file[MAX_FILENAME_LENGTH];
extern char     command_file[MAX_FILENAME_LENGTH];
extern char     temp_file[MAX_FILENAME_LENGTH];
extern char     comment_file[MAX_FILENAME_LENGTH];
extern char	log_archive_path[MAX_FILENAME_LENGTH];

extern char     macro_contact_name[MAX_CONTACTNAME_LENGTH];
extern char     macro_contact_alias[MAX_CONTACTNAME_LENGTH];
extern char	macro_host_name[MAX_HOSTNAME_LENGTH];
extern char	macro_host_alias[MAX_HOSTNAME_LENGTH];
extern char	macro_host_address[MAX_ADDRESS_LENGTH];
extern char	macro_service_description[MAX_SERVICEDESC_LENGTH];
extern char	macro_service_state[MAX_STATE_LENGTH];
extern char	macro_date_time[MAX_DATETIME_LENGTH];
extern char	macro_output[MAX_PLUGINOUTPUT_LENGTH];
extern char     macro_contact_email[MAX_EMAIL_LENGTH];
extern char     macro_contact_pager[MAX_PAGER_LENGTH];
extern char     macro_summary[MAX_COMMANDLINE_LENGTH];
extern char     macro_admin_email[MAX_EMAIL_LENGTH];
extern char     macro_admin_pager[MAX_PAGER_LENGTH];
extern char     macro_host_state[MAX_STATE_LENGTH];
extern char     macro_state_type[MAX_STATETYPE_LENGTH];
extern char     macro_current_service_attempt[MAX_ATTEMPT_LENGTH];
extern char     macro_current_host_attempt[MAX_ATTEMPT_LENGTH];
extern char     macro_argv[MAX_COMMAND_ARGUMENTS][MAX_COMMANDLINE_LENGTH];

extern char     generic_summary[MAX_COMMANDLINE_LENGTH];

extern char     global_host_event_handler[MAX_COMMANDNAME_LENGTH];
extern char     global_service_event_handler[MAX_COMMANDNAME_LENGTH];

extern int      sigshutdown;
extern int      sigrestart;

extern int      daemon_mode;

extern int	log_level;
extern int	use_syslog;
extern int	syslog_level;
extern int      log_notifications;
extern int      log_service_retries;
extern int      log_host_retries;
extern int      log_event_handlers;

extern int      sleep_time;
extern int      interval_length;
extern double   inter_check_delay;
extern int      inter_check_delay_method;
extern int      interleave_factor;
extern int      interleave_factor_method;

extern int      max_child_process_time;
extern int      child_process_timed_out;
extern int      total_orphaned_children;

extern int      zombie_cleanup_interval;
extern int      command_check_interval;
extern int      service_check_reaper_interval;

extern int      check_external_commands;

extern int      use_agressive_host_checking;

extern int      log_rotation_method;

extern int      current_comment_id;

extern int      program_mode;
extern time_t   last_mode_change;

extern time_t   last_zombie_cleanup;
extern time_t   last_command_check;
extern time_t   last_log_rotation;

extern int      verify_config;

extern int      ipc_message_queue;
extern service_message svc_msg;

extern int      max_parallel_service_checks;
extern int      currently_running_service_checks;

extern contact		*contact_list;
extern contactgroup	*contactgroup_list;
extern host		*host_list;
extern hostgroup	*hostgroup_list;
extern service		*service_list;
extern timed_event      *event_list_high;
extern timed_event      *event_list_low;
extern notification     *notification_list;
extern command          *command_list;
extern timeperiod       *timeperiod_list;

char my_system_output[MAX_INPUT_BUFFER];

#ifdef HAVE_TZNAME
extern char     *tzname[2];
#endif




/******************************************************************/
/************************ MACRO FUNCTIONS *************************/
/******************************************************************/

/* replace macros in notification commands with their values */
int process_macros(char *input_buffer,char *output_buffer,int buffer_length){
	char *temp_buffer;
	int in_macro;

#ifdef DEBUG0
	printf("process_macros() start\n");
#endif

	strcpy(output_buffer,"");

	in_macro=FALSE;

	for(temp_buffer=my_strtok(input_buffer,"$");temp_buffer!=NULL;temp_buffer=my_strtok(NULL,"$")){

		if(in_macro==FALSE){
			if(strlen(output_buffer)+strlen(temp_buffer)<MAX_COMMANDLINE_LENGTH-1){
				strncat(output_buffer,temp_buffer,buffer_length-strlen(output_buffer)-1);
				output_buffer[buffer_length-1]='\x0';
			        }
			in_macro=TRUE;
			}
		else{

			if(strlen(output_buffer)+strlen(temp_buffer)<MAX_COMMANDLINE_LENGTH-1){

				if(!strcmp(temp_buffer,"HOSTNAME"))
					strncat(output_buffer,macro_host_name,buffer_length-strlen(output_buffer)-1);

				else if(!strcmp(temp_buffer,"HOSTALIAS"))
					strncat(output_buffer,macro_host_alias,buffer_length-strlen(output_buffer)-1);

				else if(!strcmp(temp_buffer,"HOSTADDRESS"))
					strncat(output_buffer,macro_host_address,buffer_length-strlen(output_buffer)-1);

				else if(!strcmp(temp_buffer,"SERVICEDESC"))
					strncat(output_buffer,macro_service_description,buffer_length-strlen(output_buffer)-1);

				else if(!strcmp(temp_buffer,"SERVICESTATE"))
					strncat(output_buffer,macro_service_state,buffer_length-strlen(output_buffer)-1);

				else if(!strcmp(temp_buffer,"SERVICEATTEMPT"))
					strncat(output_buffer,macro_current_service_attempt,buffer_length-strlen(output_buffer)-1);

				else if(!strcmp(temp_buffer,"DATETIME"))
					strncat(output_buffer,macro_date_time,buffer_length-strlen(output_buffer)-1);

				else if(!strcmp(temp_buffer,"OUTPUT"))
					strncat(output_buffer,macro_output,buffer_length-strlen(output_buffer)-1);

				else if(!strcmp(temp_buffer,"CONTACTNAME"))
					strncat(output_buffer,macro_contact_name,buffer_length-strlen(output_buffer)-1);

				else if(!strcmp(temp_buffer,"CONTACTALIAS"))
					strncat(output_buffer,macro_contact_alias,buffer_length-strlen(output_buffer)-1);

				else if(!strcmp(temp_buffer,"CONTACTEMAIL"))
					strncat(output_buffer,macro_contact_email,buffer_length-strlen(output_buffer)-1);

				else if(!strcmp(temp_buffer,"CONTACTPAGER"))
					strncat(output_buffer,macro_contact_pager,buffer_length-strlen(output_buffer)-1);

				else if(!strcmp(temp_buffer,"ADMINEMAIL"))
					strncat(output_buffer,macro_admin_email,buffer_length-strlen(output_buffer)-1);

				else if(!strcmp(temp_buffer,"ADMINPAGER"))
					strncat(output_buffer,macro_admin_pager,buffer_length-strlen(output_buffer)-1);

				else if(!strcmp(temp_buffer,"SUMMARY"))
					strncat(output_buffer,macro_summary,buffer_length-strlen(output_buffer)-1);

				else if(!strcmp(temp_buffer,"HOSTSTATE"))
					strncat(output_buffer,macro_host_state,buffer_length-strlen(output_buffer)-1);

				else if(!strcmp(temp_buffer,"HOSTATTEMPT"))
					strncat(output_buffer,macro_current_host_attempt,buffer_length-strlen(output_buffer)-1);

				else if(!strcmp(temp_buffer,"STATETYPE"))
					strncat(output_buffer,macro_state_type,buffer_length-strlen(output_buffer)-1);

				else if(!strcmp(temp_buffer,"ARG10"))
					strncat(output_buffer,macro_argv[9],buffer_length-strlen(output_buffer)-1);
				
				else if(!strcmp(temp_buffer,"ARG11"))
					strncat(output_buffer,macro_argv[10],buffer_length-strlen(output_buffer)-1);
				
				else if(!strcmp(temp_buffer,"ARG12"))
					strncat(output_buffer,macro_argv[11],buffer_length-strlen(output_buffer)-1);
				
				else if(!strcmp(temp_buffer,"ARG13"))
					strncat(output_buffer,macro_argv[12],buffer_length-strlen(output_buffer)-1);
				
				else if(!strcmp(temp_buffer,"ARG14"))
					strncat(output_buffer,macro_argv[13],buffer_length-strlen(output_buffer)-1);
				
				else if(!strcmp(temp_buffer,"ARG15"))
					strncat(output_buffer,macro_argv[14],buffer_length-strlen(output_buffer)-1);
				
				else if(!strcmp(temp_buffer,"ARG16"))
					strncat(output_buffer,macro_argv[15],buffer_length-strlen(output_buffer)-1);
	
				else if(!strcmp(temp_buffer,"ARG1"))
					strncat(output_buffer,macro_argv[0],buffer_length-strlen(output_buffer)-1);
				
				else if(!strcmp(temp_buffer,"ARG2"))
					strncat(output_buffer,macro_argv[1],buffer_length-strlen(output_buffer)-1);
		
				else if(!strcmp(temp_buffer,"ARG3"))
					strncat(output_buffer,macro_argv[2],buffer_length-strlen(output_buffer)-1);

				else if(!strcmp(temp_buffer,"ARG4"))
					strncat(output_buffer,macro_argv[3],buffer_length-strlen(output_buffer)-1);
				
				else if(!strcmp(temp_buffer,"ARG5"))
					strncat(output_buffer,macro_argv[4],buffer_length-strlen(output_buffer)-1);
				
				else if(!strcmp(temp_buffer,"ARG6"))
					strncat(output_buffer,macro_argv[5],buffer_length-strlen(output_buffer)-1);
				
				else if(!strcmp(temp_buffer,"ARG7"))
					strncat(output_buffer,macro_argv[6],buffer_length-strlen(output_buffer)-1);
				
				else if(!strcmp(temp_buffer,"ARG8"))
					strncat(output_buffer,macro_argv[7],buffer_length-strlen(output_buffer)-1);
				
				else if(!strcmp(temp_buffer,"ARG9"))
					strncat(output_buffer,macro_argv[8],buffer_length-strlen(output_buffer)-1);
			
				else{
					strncat(output_buffer,"$",buffer_length-strlen(output_buffer)-1);
					output_buffer[buffer_length-1]='\x0';
					strncat(output_buffer,(temp_buffer==NULL)?"":temp_buffer,buffer_length-strlen(output_buffer)-1);
					output_buffer[buffer_length-1]='\x0';
					strncat(output_buffer,"$",buffer_length-strlen(output_buffer)-1);
				        }

				output_buffer[buffer_length-1]='\x0';
				}

			in_macro=FALSE;
			}
		}

#ifdef DEBUG0
	printf("process_macros() end\n");
#endif

	return OK;
	}


/* grab macros that are specific to a particular service */
int grab_service_macros(service *svc){
	time_t t;
	
#ifdef DEBUG0
	printf("grab_service_macros() start\n");
#endif

	/* get the service description */
	strncpy(macro_service_description,svc->description,sizeof(macro_service_description));
	macro_service_description[sizeof(macro_service_description)-1]='\x0';

	/* get the plugin output */
	strncpy(macro_output,svc->plugin_output,sizeof(macro_output));
	macro_output[sizeof(macro_output)-1]='\x0';

	/* get the service state */
	if(svc->current_state==STATE_OK)
		strcpy(macro_service_state,"RECOVERY");
	else if(svc->current_state==STATE_WARNING)
		strcpy(macro_service_state,"WARNING");
	else if(svc->current_state==STATE_CRITICAL)
		strcpy(macro_service_state,"CRITICAL");
	else
		strcpy(macro_service_state,"UNKNOWN");

	/* get the current date/time */
	time(&t);
	get_datetime_string(&t,macro_date_time,sizeof(macro_date_time));

	/* get the current service check attempt macro */
	snprintf(macro_current_service_attempt,sizeof(macro_current_service_attempt),"%d",svc->current_attempt);
	macro_current_service_attempt[sizeof(macro_current_service_attempt)-1]='\x0';

	strip(macro_service_description);
	strip(macro_output);
	strip(macro_service_state);
	strip(macro_date_time);
	strip(macro_current_service_attempt);

#ifdef DEBUG0
	printf("grab_service_macros() end\n");
#endif

	return OK;
        }


/* grab macros that are specific to a particular host */
int grab_host_macros(host *hst){
	time_t t;

#ifdef DEBUG0
	printf("grab_host_macros() start\n");
#endif

	/* get the host name */
	strncpy(macro_host_name,hst->name,sizeof(macro_host_name));
	macro_host_name[sizeof(macro_host_name)-1]='\x0';
	
	/* get the host alias */
	strncpy(macro_host_alias,hst->alias,sizeof(macro_host_alias));
	macro_host_alias[sizeof(macro_host_alias)-1]='\x0';

	/* get the host address */
	strncpy(macro_host_address,hst->address,sizeof(macro_host_address));
	macro_host_address[sizeof(macro_host_address)-1]='\x0';

	/* get the host state */
	if(hst->status==HOST_DOWN)
		strcpy(macro_host_state,"DOWN");
	else if(hst->status==HOST_UNREACHABLE)
		strcpy(macro_host_state,"UNREACHABLE");
	else
		strcpy(macro_host_state,"UP");

	/* get the plugin output */
	strncpy(macro_output,hst->plugin_output,sizeof(macro_output));
	macro_output[sizeof(macro_output)-1]='\x0';

	/* get the host current attempt */
	snprintf(macro_current_host_attempt,sizeof(macro_current_host_attempt),"%d",hst->current_attempt);
	macro_current_host_attempt[sizeof(macro_current_host_attempt)-1]='\x0';

	/* get the current date/time */
	time(&t);
	get_datetime_string(&t,macro_date_time,sizeof(macro_date_time));

	strip(macro_host_name);
	strip(macro_host_alias);
	strip(macro_host_address);
	strip(macro_host_state);
	strip(macro_output);
	strip(macro_current_host_attempt);
	strip(macro_date_time);

#ifdef DEBUG0
	printf("grab_host_macros() end\n");
#endif

	return OK;
        }


/* grab macros that are specific to a particular contact */
int grab_contact_macros(contact *cntct){
	time_t t;

#ifdef DEBUG0
	printf("grab_contact_macros() start\n");
#endif

	/* get the name */
	strncpy(macro_contact_name,cntct->name,sizeof(macro_contact_name));
	macro_contact_name[sizeof(macro_contact_name)-1]='\x0';

	/* get the alias */
	strncpy(macro_contact_alias,cntct->alias,sizeof(macro_contact_alias));
	macro_contact_alias[sizeof(macro_contact_alias)-1]='\x0';

	/* get the email address */
	strncpy(macro_contact_email,cntct->email,sizeof(macro_contact_email));
	macro_contact_email[sizeof(macro_contact_email)-1]='\x0';

	/* get the pager number */
	strncpy(macro_contact_pager,cntct->pager,sizeof(macro_contact_pager));
	macro_contact_pager[sizeof(macro_contact_pager)-1]='\x0';

	/* get the current date/time */
	time(&t);
	get_datetime_string(&t,macro_date_time,sizeof(macro_date_time));

	strip(macro_contact_name);
	strip(macro_contact_alias);
	strip(macro_contact_email);
	strip(macro_contact_pager);
	strip(macro_date_time);

#ifdef DEBUG0
	printf("grab_contact_macros() end\n");
#endif

	return OK;
        }


/* clear all macros that are not "constant" (i.e. they change throughout the course of monitoring) */
int clear_volatile_macros(void){
	int x;

#ifdef DEBUG0
	printf("clear_volatile_macros() start\n");
#endif
	
	strcpy(macro_contact_name,"");
	strcpy(macro_contact_alias,"");
	strcpy(macro_contact_email,"");
	strcpy(macro_contact_pager,"");

	strcpy(macro_host_name,"");
	strcpy(macro_host_alias,"");
	strcpy(macro_host_address,"");
	strcpy(macro_host_state,"");
	strcpy(macro_current_host_attempt,"");

	strcpy(macro_service_description,"");
	strcpy(macro_service_state,"");
	strcpy(macro_current_service_attempt,"");

	strcpy(macro_state_type,"");

	strcpy(macro_date_time,"");

	for(x=0;x<MAX_COMMAND_ARGUMENTS;x++)
		strcpy(macro_argv[x],"");

#ifdef DEBUG0
	printf("clear_volatile_macros() end\n");
#endif

	return OK;
        }


/* clear macros that are constant (i.e. they do NOT change during monitoring) */
int clear_nonvolatile_macros(void){

#ifdef DEBUG0
	printf("clear_nonvolatile_macros() start\n");
#endif

	strcpy(macro_admin_email,"");
	strcpy(macro_admin_pager,"");
	strcpy(macro_summary,"");

#ifdef DEBUG0
	printf("clear_nonvolatile_macros() end\n");
#endif

	return OK;
        }




/******************************************************************/
/******************** SYSTEM COMMAND FUNCTIONS ********************/
/******************************************************************/


/* executes a system command - used for service checks and notifications */
int my_system(char *command){
        int pid;
	int status;
	int wait_result;
	int result;
	extern int errno;
	extern char **environ;
	char buffer[MAX_INPUT_BUFFER];
	time_t start_time;
	time_t current_time;
	char *argv[4];

#ifdef DEBUG0
	printf("my_system() start\n");
#endif

	/* if no command was passed, return with no error */
	if(command==NULL)
	        return STATE_OK;

	/* fork */
	pid=fork();

	/* return an error if we couldn't fork */
	if(pid==-1){

		sprintf(buffer,"Warning: fork() in my_system() failed for command \"%s\"\n",command);
		write_to_log(buffer);
#ifdef DEBUG1
		printf("\t%s",buffer);
#endif

	        return STATE_UNKNOWN;  
	        }

	/* execute the command in the child process */
        if (pid==0){

		/* become process group leader */
		setpgid(0,0);

		/* Added 08/16/1999 - free allocated memory */
		free_memory();

		/* Added 10/10/1999 - reset signal handling */
		reset_sighandler();

		/* command line executed will be '/bin/sh -c <command>' */
		argv[0]="/bin/sh";
		argv[1]="-c";
		argv[2]=command;
		argv[3]=0;

		/* execute the command, replacing the child process */
		execve("/bin/sh",argv,environ);

		sprintf(buffer,"Warning: execve() in my_system() failed for command \"%s\"\n",command);
		write_to_log(buffer);
#ifdef DEBUG1
		printf("\t%s",buffer);
#endif

		/* this code should not be reached */
		/* changed exit() to _exit() - 08/08/1999 */
		_exit(STATE_UNKNOWN);
	        }

	/* get the time the child process was started */
	time(&start_time);

	/* wait for the command to finish */
	while(1){

		/* see if the child process has exited, but don't wait for it to exit */
		wait_result=waitpid(pid,&status,WNOHANG | WUNTRACED);

		/* the child process finished executing */
		if(wait_result==-1)
			break;


		/* the child process has not exited yet - it is still running... */
		else if(wait_result==0){

			/* get the current time */
			time(&current_time);

			/* make sure we haven't waited too long for the child process to finish running... */
			if(current_time-start_time>max_child_process_time){

				sprintf(buffer,"Warning: A timeout occurred with process having PID=%d while trying to execute \"%s\"\n",pid,command);
				write_to_log(buffer);
#ifdef DEBUG1
				printf("\t%s",buffer);
#endif

				/* wake up child process if it is stopped... */
				killpg(pid,SIGCONT);

				/* send child process an interrupt signal... */
				killpg(pid,SIGINT);

				/* sleep for a second and give the child some time to terminate... */
				sleep(1);

				/* kill the child process... */
				killpg(pid,SIGKILL);

				/* attempt to clear the zombie status of the child process, but don't wait around for it
				 to terminate if it hasn't already... */
				wait_result=waitpid(pid,&status,WNOHANG | WUNTRACED);

				/* we couldn't kill the child process for some reason... */
				if(wait_result!=pid){
					snprintf(buffer,sizeof(buffer),"Warning: Could not kill child process with PID=%d.  Zombie process cleanup should take care of this later...\n",pid);
					buffer[sizeof(buffer)-1]='\x0';
					write_to_log(buffer);
#ifdef DEBUG1
					printf("\t%s",buffer);
#endif
				        /* increment the number of child processes that are going to be in zombie states */
					total_orphaned_children++;

					/* update status log with program info */
					update_status_log_with_program();
				        }

				/* else log the successful kill.. */
				else{
					snprintf(buffer,sizeof(buffer),"Successfully killed child process with PID=%d!\n",pid);
					buffer[sizeof(buffer)-1]='\x0';
					write_to_log(buffer);
#ifdef DEBUG1
					printf("\t%s",buffer);
#endif
				        }

				/* set the early kill flag */
				child_process_timed_out=TRUE;

				return STATE_UNKNOWN;
			        }
			
			/* sleep for a second so we don't hog cpu time */
			sleep(1);
		        }

	        }

	/* get the exit code returned from the program */
	result=WEXITSTATUS(status);

#ifdef DEBUG1
	printf("my_system() end with status = %d,result=%d\n",status,result);
#endif

	/* if the command couldn't be found, write an error to the log file */
	if(result==126||result==127){

		snprintf(buffer,sizeof(buffer),"Warning: Command \"%s\" not found\n",command);
		buffer[sizeof(buffer)-1]='\x0';
		write_to_log(buffer);
#ifdef DEBUG1
		printf("\t%s",buffer);
#endif
	        }

	/* check the bounds for the return code */
	if(result<0||result>2)
		return STATE_UNKNOWN;

	return result;
        }


/* given a "raw" command, return the "expanded" or "whole" command line */
void get_raw_command_line(char *cmd,char *raw_command,int buffer_length){
	char temp_buffer[MAX_COMMANDLINE_LENGTH];
	char *buffer;
	command *temp_command;
	int x;

#ifdef DEBUG0
	printf("get_raw_command_line() start\n");
#endif
#ifdef DEBUG1
	printf("\tInput: %s\n",cmd);
#endif

	/* initialize the command */
	strcpy(raw_command,"");

	/* if we were handed a NULL string, throw it right back */
	if(cmd==NULL){
		
#ifdef DEBUG1
		printf("\tRaw command is NULL!\n");
#endif

		return;
	        }

	/* clear the old command arguments */
	for(x=0;x<MAX_COMMAND_ARGUMENTS;x++)
		strcpy(macro_argv[x],"");

	/* if the command is enclosed in quotes, this *is* the command we should run, don't do a lookup */
	if(cmd[0]=='\"'){
		strncpy(raw_command,cmd+1,buffer_length);
		raw_command[buffer_length-1]='\x0';
		strip(raw_command);

		/* strip the trailing quote if its there... */
		if(raw_command[strlen(raw_command)-1]=='\"')
			raw_command[strlen(raw_command)-1]='\x0';
	        }

	/* else lookup the command */
	else{

		strcpy(temp_buffer,cmd);

		/* get the command name */
		buffer=my_strtok(temp_buffer,"!");

		/* the buffer should never be NULL, but just in case... */
		if(buffer==NULL){
			strcpy(raw_command,"");
			return;
		        }
		else{
			strncpy(raw_command,buffer,buffer_length);
			raw_command[buffer_length-1]='\x0';
		        }

		/* get the arguments */
		for(x=0;x<MAX_COMMAND_ARGUMENTS;x++){
			buffer=my_strtok(NULL,"!");
			if(buffer==NULL)
				break;
			strncpy(macro_argv[x],buffer,sizeof(macro_argv[x]));
			macro_argv[x][sizeof(macro_argv[x])-1]='\x0';
			strip(buffer);
		        }

		/* find the command used to check this service */
		temp_command=NULL;
		temp_command=find_command(raw_command,temp_command);

		/* error if we couldn't find the command */
		if(temp_command==NULL)
			return;

		strncpy(raw_command,temp_command->command_line,buffer_length);
		raw_command[buffer_length-1]='\x0';
		strip(raw_command);
	        }

#ifdef DEBUG1
	printf("\tOutput: %s\n",raw_command);
#endif
#ifdef DEBUG0
	printf("get_raw_command_line() end\n");
#endif

	return;
        }





/******************************************************************/
/************************* TIME FUNCTIONS *************************/
/******************************************************************/


/* see if the specified time falls into a valid time range in the given time period */
int check_time_against_period(unsigned long check_time,char *period_name){
	timeperiod *temp_period;
	timerange *temp_range;
	unsigned long midnight_today;
	struct tm *t;

#ifdef DEBUG0
	printf("check_time_against_period() start\n");
#endif
	
	/* if no period name was specified, assume the time is good */
	if(!strcmp(period_name,""))
	   return 1;

	/* if period could not be found, assume the time is good */
	temp_period=find_timeperiod(period_name,NULL);
	if(temp_period==NULL)
		return 1;

	t=localtime((time_t *)&check_time);

	/* calculate the start of the day (midnight, 00:00 hours) */
	t->tm_sec=0;
	t->tm_min=0;
	t->tm_hour=0;
	midnight_today=(unsigned long)mktime(t);

	for(temp_range=temp_period->days[t->tm_wday];temp_range!=NULL;temp_range=temp_range->next){

		/* if the user-specified time falls in this range, return with a positive result */
		if(check_time>=(midnight_today+temp_range->range_start) && check_time<=(midnight_today+temp_range->range_end))
			return 1;
	        }

#ifdef DEBUG0
	printf("check_time_against_period() end\n");
#endif

	return 0;
        }


/* given a preferred time, get the next valid time within a time period */ 
void get_next_valid_time(unsigned long preferred_time,unsigned long *valid_time, char *period_name){
	timeperiod *temp_period;
	timerange *temp_timerange;
	unsigned long midnight_today;
	struct tm *t;
	int day;
	int today;
	int weekday;
	unsigned long earliest_next_valid_time=0L;

#ifdef DEBUG0
	printf("get_next_valid_time() start\n");
#endif

#ifdef DEBUG1
	printf("\tPreferred Time: %lu\n",preferred_time);
#endif

	/* if the preferred time is valid today, go with it */
	if(check_time_against_period(preferred_time,period_name))
		*valid_time=preferred_time;

	/* else find the next available time */
	else{

		/* find the time period - if we can't find it, go with the preferred time */
		temp_period=find_timeperiod(period_name,NULL);
		if(temp_period==NULL){
			*valid_time=preferred_time;
			return;
		        }


		t=localtime((time_t *)&preferred_time);

		/* calculate the start of the day (midnight, 00:00 hours) */
		t->tm_sec=0;
		t->tm_min=0;
		t->tm_hour=0;
		midnight_today=(unsigned long)mktime(t);

		today=t->tm_wday;

		/* check a one week rotation of time */
		for(day=1;day<=7;day++){

			weekday=today+day;
			if(weekday>=7)
				weekday-=7;

			/* check all time ranges for this day of the week */
			for(temp_timerange=temp_period->days[weekday];temp_timerange!=NULL;temp_timerange=temp_timerange->next){

				/* we're looking for the earliest possible start time for this day */
				if(earliest_next_valid_time==0L || temp_timerange->range_start<earliest_next_valid_time)
					earliest_next_valid_time=temp_timerange->range_start;
			        }

			/* break out of the loop if we found a time already */
			if(earliest_next_valid_time!=0L)
				break;
		        }

		/* if we couldn't find a time period (there must be none defined), pass back the preferred time */
		if(earliest_next_valid_time==0L)
			*valid_time=preferred_time;

		/* else we must calculate the number of seconds to use... */
		else
			*valid_time=(unsigned long)(midnight_today+(day*3600*24)+earliest_next_valid_time);
	        }

#ifdef DEBUG1
	printf("\tNext Valid Time: %lu\n",*valid_time);
#endif

#ifdef DEBUG0
	printf("get_next_valid_time() end\n");
#endif

	return;
        }
	
	 

/* given a date/time in time_t format, produce a corresponding date/time string, including timezone */
void get_datetime_string(unsigned long *raw_time,char *buffer,int buffer_length){
	time_t t;
	struct tm *tm_ptr;
	int day;
	int hour;
	int minute;
	int second;
	int year;
	char *weekdays[7]={"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
	char *months[12]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sept","Oct","Nov","Dec"};
	char *timezone="";

	if(raw_time==NULL)
		time(&t);
	else 
		t=(time_t)(*raw_time);

	tm_ptr=localtime(&t);

	hour=tm_ptr->tm_hour;
	minute=tm_ptr->tm_min;
	second=tm_ptr->tm_sec;
	day=tm_ptr->tm_mday;
	year=tm_ptr->tm_year+1900;

#ifdef HAVE_TM_ZONE
	timezone=(char *)tm_ptr->tm_zone;
#else
	timezone=(char *)tzname[1];
#endif

	snprintf(buffer,buffer_length,"%s %s %d %02d:%02d:%02d %s %d",weekdays[tm_ptr->tm_wday],months[tm_ptr->tm_mon],day,hour,minute,second,timezone,year);
	buffer[buffer_length-1]='\x0';

	return;
        }



/* get the next time to schedule a log rotation */
void get_next_log_rotation_time(time_t *run_time){
	time_t current_time;
	unsigned long midnight_today;
	unsigned long midnight_tonight;
	unsigned long midnight_weekend;
	unsigned long midnight_month;
	unsigned long past_hour;
	unsigned long next_hour;
	struct tm *t;

#ifdef DEBUG0
	printf("get_next_log_rotation_time() start\n");
#endif

	time(&current_time);

	t=localtime(&current_time);

	/* calculate the start of the hour */
	t->tm_sec=0;
	t->tm_min=0;
	past_hour=(unsigned long)mktime(t);

	next_hour=past_hour+(60*60);

	/* calculate the start of this day (midnight, 00:00 hours) */
	t->tm_hour=0;
	midnight_today=(unsigned long)mktime(t);

	/* calculate the start of tomorrow (midnight, 00:00 hours) */
	midnight_tonight=midnight_today+(60*60*24);

	/* calculate the start of the day (midnight, 00:00 hours) for the next Sunday morning (Saturday evening)*/
	midnight_weekend=midnight_today+(60*60*24*(7-t->tm_wday));
	if(midnight_weekend==midnight_today)
		midnight_weekend+=(60*60*24*7);

	/* calculate the start of the day (midnight, 00:00 hours) for the first day next month */
	t->tm_mday=1;
	if(t->tm_mon==11){
		t->tm_mon=1;
		t->tm_year+=1;
	        }
	else
		t->tm_mon++;
	midnight_month=(unsigned long)mktime(t);


	switch(log_rotation_method){
	case LOG_ROTATION_HOURLY:
		*run_time=(time_t)next_hour;
		break;
	case LOG_ROTATION_DAILY:
		*run_time=(time_t)midnight_tonight;
		break;
	case LOG_ROTATION_WEEKLY:
		*run_time=(time_t)midnight_weekend;
		break;
	case LOG_ROTATION_MONTHLY:
		*run_time=(time_t)midnight_month;
		break;
	default:
		break;
	        }

#ifdef DEBUG1
	printf("\tNext Log Rotation Time: %lu\n",(unsigned long)*run_time);
#endif

#ifdef DEBUG0
	printf("get_next_log_rotation_time() end\n");
#endif

	return;
        }


/******************************************************************/
/********************* PROGRAM MODE FUNCTIONS *********************/
/******************************************************************/

/* switch to standby or active mode */
void set_program_mode(int mode){
	char temp_buffer[MAX_INPUT_BUFFER];

#ifdef DEBUG0
	printf("set_program_mode() start\n");
#endif

	/* set the program mode variable */
	if(mode==STANDBY_MODE)
		program_mode=STANDBY_MODE;
	else
		program_mode=ACTIVE_MODE;

	/* update the last mode change time */
	last_mode_change=time(NULL);

	/* update the status log */
	update_status_log_with_program();

	snprintf(temp_buffer,sizeof(temp_buffer),"Entering %s mode...\n",(mode==ACTIVE_MODE)?"active":"standby");
	temp_buffer[sizeof(temp_buffer)-1]='\x0';
	write_to_log(temp_buffer);

#ifdef DEBUG0
	printf("set_program_mode() end\n");
#endif

	return;
        }




/******************************************************************/
/******************** SIGNAL HANDLER FUNCTIONS ********************/
/******************************************************************/


/* trap signals so we can exit gracefully */
void setup_sighandler(void){

#ifdef DEBUG0
	printf("setup_sighandler() start\n");
#endif

	/* reset the shutdown flag */
	sigshutdown=FALSE;

	/* remove buffering from stderr, stdin, and stdout */
	setbuf(stdin,(char *)NULL);
	setbuf(stdout,(char *)NULL);
	setbuf(stderr,(char *)NULL);

	/* initialize signal handling */
	signal(SIGPIPE,SIG_IGN);
	signal(SIGINT,sighandler);
	signal(SIGQUIT,sighandler);
	signal(SIGTERM,sighandler);
	signal(SIGBUS,sighandler);
	signal(SIGFPE,sighandler);
	signal(SIGILL,sighandler);
	signal(SIGHUP,sighandler);
	signal(SIGABRT,sighandler);	
	signal(SIGSEGV,sighandler);
	signal(SIGKILL,sighandler);

	/* removed child signal handler 08/08/1999 */
	/*signal(SIGCHLD,sighandler);*/

#ifdef DEBUG0
	printf("setup_sighandler() end\n");
#endif

	return;
        }


/* reset signal handling... */
void reset_sighandler(void){

#ifdef DEBUG0
	printf("reset_sighandler() start\n");
#endif

	/* set signal handling to default actions */
	signal(SIGPIPE,SIG_DFL);
	signal(SIGINT,SIG_DFL);
	signal(SIGQUIT,SIG_DFL);
	signal(SIGTERM,SIG_DFL);
	signal(SIGBUS,SIG_DFL);
	signal(SIGFPE,SIG_DFL);
	signal(SIGILL,SIG_DFL);
	signal(SIGHUP,SIG_DFL);
	signal(SIGABRT,SIG_DFL);	
	signal(SIGSEGV,SIG_DFL);
	signal(SIGKILL,SIG_DFL);

#ifdef DEBUG0
	printf("reset_sighandler() end\n");
#endif

	return;
        }


/* handle signals */
void sighandler(int sig){
	static char *sigs[]={"EXIT","HUP","INT","QUIT","ILL","TRAP","ABRT","BUS","FPE","KILL","USR1","SEGV","USR2","PIPE","ALRM","TERM","STKFLT","CHLD","CONT","STOP","TSTP","TTIN","TTOU","URG","XCPU","XFSZ","VTALRM","PROF","WINCH","IO","PWR","UNUSED","ZERR","DEBUG",(char *)NULL};
	int i;
	char temp_buffer[MAX_INPUT_BUFFER];

	/* if shutdown is already true, we're in a signal trap loop! */
	if(sigshutdown==TRUE)
		exit(ERROR);

	if(sig<0)
		sig=-sig;

	for(i=0;sigs[i]!=(char *)NULL;i++);

	sig%=i;

	if(sig==1){

		sigrestart=TRUE;

		sprintf(temp_buffer,"Caught SIGHUP, restarting...\n");
		write_to_log(temp_buffer);
		write_to_syslog(temp_buffer);
#ifdef DEBUG2
		printf("%s\n",temp_buffer);
#endif
	        }

	else if(sig<16){

		sigshutdown=TRUE;

		sprintf(temp_buffer,"Caught SIG%s, shutting down...\n",sigs[sig]);
		write_to_log(temp_buffer);
		write_to_syslog(temp_buffer);

#ifdef DEBUG2
		printf("%s\n",temp_buffer);
#endif

		/* remove the lock file if we're in daemon mode */
		if(daemon_mode==TRUE)
			unlink(LOCKFILE);

	        }

	return;
        }


/* handle timeouts when executing service checks */
void service_check_sighandler(int sig){

	/* write plugin check results to message queue */
	strncpy(svc_msg.output,"(Service Check Timed Out)",sizeof(svc_msg.output)-1);
	svc_msg.output[sizeof(svc_msg.output)-1]='\x0';
	svc_msg.return_code=STATE_CRITICAL;
	svc_msg.exited_ok=TRUE;
	send_svc_message(ipc_message_queue,&svc_msg);

	/* force the child process (service check) to exit... */
	exit(STATE_CRITICAL);
        }



/******************************************************************/
/************************ DAEMON FUNCTIONS ************************/
/******************************************************************/

int daemon_init(void){
	pid_t	pid;
	int pidno;
	int lockfile;
	struct rlimit limit;
	FILE *fp;

	chdir("/");		/* change working directory */

	umask(0);		/* clear our file mode creation mask */

	lockfile = open(LOCKFILE, O_CREAT | O_EXCL, S_IWUSR | S_IRUSR | S_IRGRP | S_IROTH);

	if(lockfile==-1){
		if(errno==EEXIST){
			fp=fopen(LOCKFILE,"r");
			fscanf(fp,"%d",&pidno);
			fclose(fp);
			pid=(pid_t)pidno;
			if(pid==getpid())    /* SIGHUP */
				return 0;
			printf("Lockfile already exists (process %d found %d in Lockfile)\n",(int)getpid(),(int)pid);
		        }
		else if (errno==EISDIR)
			printf("%s is a directory\n",LOCKFILE);
		else if (errno==EACCES)
			printf("You do not have permission to write to %s\n",LOCKFILE);
		else if (errno==ENAMETOOLONG)
			printf("The filename is too long: %s\n",LOCKFILE);
		else if (errno==ENOENT)
			printf("%s does not exist (ENOENT)\n",LOCKFILE);
		else if (errno==ENOTDIR)
			printf("%s does not exist (ENOTDIR)\n",LOCKFILE);
		else if (errno==ENXIO)
			printf("Cannot write to special file\n");
		else if (errno==ENODEV)
			printf("Cannot write to device\n");
		else if (errno==EROFS)
			printf("%s is ona read-only file system\n",LOCKFILE);
		else if (errno==ETXTBSY)
			printf("%s is a currently running program\n",LOCKFILE);
		else if (errno==EFAULT)
			printf("%s is outside address space\n",LOCKFILE);
		else if (errno==ELOOP)
			printf("Too many symbolic links\n");
		else if (errno==ENOSPC)
			printf("No space on device\n");
		else if (errno==ENOMEM)
			printf("Insufficient kernel memory\n");
		else if (errno==EMFILE)
			printf("Too many files open in process\n");
		else if (errno==ENFILE)
			printf("Too many files open on system\n");
		exit(1);
	        }

	close(lockfile);

	/* exit on errors... */
	if((pid=fork())<0)
		return(-1);

	/* parent process goes away.. */
	else if((int)pid != 0)
		exit(0);

	/* child continues... */

	/* child becomes session leader.. */
	setsid();		

	/* prevent daemon from dumping a core file... */
	getrlimit(RLIMIT_CORE,&limit);
	limit.rlim_cur=0;
	setrlimit(RLIMIT_CORE,&limit);

	/* write PID to lockfile... */
	pid=getpid();
	fp=fopen(LOCKFILE,"w");
	fprintf(fp,"%d",(int)pid);
	fclose(fp);
	
	return(0);
	}




/******************************************************************/
/************************ STRING FUNCTIONS ************************/
/******************************************************************/

/* strip newline, carriage return, and tab characters from end of a string */
void strip(char *buffer){
	int lastchar;

	lastchar=strlen(buffer)-1;

	while((buffer[lastchar]=='\n'||buffer[lastchar]=='\r'||buffer[lastchar]==' '||buffer[lastchar]=='\t') && lastchar>=0){
		buffer[lastchar]='\x0';
		lastchar=strlen(buffer)-1;
	        }

	return;
	}


/* fix the problem with strtok() skipping empty options between tokens */	
char *my_strtok(char *buffer,char *tokens){
	char *token_position;
	char *sequence_head;

	if(buffer!=NULL){
		if(original_my_strtok_buffer!=NULL)
			free(original_my_strtok_buffer);
		my_strtok_buffer=malloc(strlen(buffer)+1);
		if(my_strtok_buffer==NULL)
			return NULL;
		original_my_strtok_buffer=my_strtok_buffer;
		strcpy(my_strtok_buffer,buffer);
	        }
	
	sequence_head=my_strtok_buffer;

	if(sequence_head[0]=='\x0')
		return NULL;
	
	token_position=index(my_strtok_buffer,tokens[0]);

	if(token_position==NULL){
		my_strtok_buffer=index(my_strtok_buffer,'\x0');
		return sequence_head;
	        }

	token_position[0]='\x0';
	my_strtok_buffer=token_position+1;

	return sequence_head;

        }



/* fixes compiler problems under Solaris, since strsep() isn't included */
/* this code is taken from the glibc source */
char *my_strsep (char **stringp, const char *delim){
	char *begin, *end;

	begin = *stringp;
	if (begin == NULL)
		return NULL;

	/* A frequent case is when the delimiter string contains only one
	   character.  Here we don't need to call the expensive `strpbrk'
	   function and instead work using `strchr'.  */
	if(delim[0]=='\0' || delim[1]=='\0'){
		char ch = delim[0];

		if(ch=='\0')
			end=NULL;
		else{
			if(*begin==ch)
				end=begin;
			else
				end=strchr(begin+1,ch);
			}
		}

	else
		/* Find the end of the token.  */
		end = strpbrk (begin, delim);

	if(end){

		/* Terminate the token and set *STRINGP past NUL character.  */
		*end++='\0';
		*stringp=end;
		}
	else
		/* No more delimiters; this is the last token.  */
		*stringp=NULL;

	return begin;
	}



/* fixes a compiler warning under HP-UX */
char *my_index(char *buffer, char token){
	int index;

	if(buffer==NULL)
		return NULL;

	for(index=0;buffer[index]!='\x0'&&buffer[index]!=token;index++);

	return buffer+index;
        }





/******************************************************************/
/*********************** CLEANUP FUNCTIONS ************************/
/******************************************************************/

/* do some cleanup before we exit */
void cleanup(void){

#ifdef DEBUG0
	printf("cleanup() start\n");
#endif

	/* free all allocated memory */
	free_memory();

	/* reset global variables to default values */
	reset_variables();

	/* clear all macros */
	clear_volatile_macros();
	clear_nonvolatile_macros();

	/* remove the IPC message queue */
	if(verify_config==FALSE)
		remove_message_queue(ipc_message_queue);


#ifdef DEBUG0
	printf("cleanup() end\n");
#endif

	return;
	}


/* free the memory allocated to the linked lists */
void free_memory(void){
	timeperiod *this_timeperiod=NULL;
	timeperiod *next_timeperiod=NULL;
	timerange *this_timerange=NULL;
	timerange *next_timerange=NULL;
	host *this_host=NULL;
	host *next_host=NULL;
	hostgroup *this_hostgroup=NULL;
	hostgroup *next_hostgroup=NULL;
	hostgroupmember *this_hostgroupmember=NULL;
	hostgroupmember *next_hostgroupmember=NULL;
	contact	*this_contact=NULL;
	contact *next_contact=NULL;
	contactgroup *this_contactgroup=NULL;
	contactgroup *next_contactgroup=NULL;
	contactgroupmember *this_contactgroupmember=NULL;
	contactgroupmember *next_contactgroupmember=NULL;
	service *this_service=NULL;
	service *next_service=NULL;
	timed_event *this_event=NULL;
	timed_event *next_event=NULL;
	command *this_command=NULL;
	command *next_command=NULL;
	commandsmember *this_commandsmember=NULL;
	commandsmember *next_commandsmember=NULL;
	int day;

#ifdef DEBUG0
	printf("free_memory() start\n");
#endif

	/* free memory for the timeperiod list */
	this_timeperiod=timeperiod_list;
	while(this_timeperiod!=NULL){

		/* free the time ranges contained in this timeperiod */
		for(day=0;day<7;day++){

			for(this_timerange=this_timeperiod->days[day];this_timerange!=NULL;this_timerange=next_timerange){
				next_timerange=this_timerange->next;
				free(this_timerange);
			        }
		        }

		next_timeperiod=this_timeperiod->next;
		free(this_timeperiod);
		this_timeperiod=next_timeperiod;
		}

	/* reset the host pointer */
	timeperiod_list=NULL;

#ifdef DEBUG1
	printf("\timeperiod_list freed\n");
#endif

	/* free memory for the host list */
	this_host=host_list;
	while(this_host!=NULL){
		next_host=this_host->next;
		free(this_host);
		this_host=next_host;
		}

	/* reset the host pointer */
	host_list=NULL;

#ifdef DEBUG1
	printf("\thost_list freed\n");
#endif

	/* free memory for the host group list */
	this_hostgroup=hostgroup_list;
	while(this_hostgroup!=NULL){

		/* free memory for the group members */
		this_hostgroupmember=this_hostgroup->members;
		while(this_hostgroupmember!=NULL){
			next_hostgroupmember=this_hostgroupmember->next;
			free(this_hostgroupmember);
			this_hostgroupmember=next_hostgroupmember;
		        }

		next_hostgroup=this_hostgroup->next;
		free(this_hostgroup);
		this_hostgroup=next_hostgroup;
		}

	/* reset the hostgroup pointer */
	hostgroup_list=NULL;

#ifdef DEBUG1
	printf("\thostgroup_list freed\n");
#endif

	/* free memory for the contact list */
	this_contact=contact_list;
	while(this_contact!=NULL){
	  
		/* free memory for the host notification commands */
		this_commandsmember=this_contact->host_notification_commands;
		while(this_commandsmember!=NULL){
			next_commandsmember=this_commandsmember->next;
			free(this_commandsmember);
			this_commandsmember=next_commandsmember;
		        }

		/* free memory for the service notification commands */
		this_commandsmember=this_contact->service_notification_commands;
		while(this_commandsmember!=NULL){
			next_commandsmember=this_commandsmember->next;
			free(this_commandsmember);
			this_commandsmember=next_commandsmember;
		        }

		next_contact=this_contact->next;
		free(this_contact);
		this_contact=next_contact;
		}

	/* reset the contact pointer */
	contact_list=NULL;

#ifdef DEBUG1
	printf("\tcontact_list freed\n");
#endif

	/* free memory for the contact group list */
	this_contactgroup=contactgroup_list;
	while(this_contactgroup!=NULL){

		/* free memory for the group members */
		this_contactgroupmember=this_contactgroup->members;
		while(this_contactgroupmember!=NULL){
			next_contactgroupmember=this_contactgroupmember->next;
			free(this_contactgroupmember);
			this_contactgroupmember=next_contactgroupmember;
		        }

		next_contactgroup=this_contactgroup->next;
		free(this_contactgroup);
		this_contactgroup=next_contactgroup;
		}

	/* reset the contactgroup pointer */
	contactgroup_list=NULL;

#ifdef DEBUG1
	printf("\tcontactgroup_list freed\n");
#endif

	/* free memory for the service list */
	this_service=service_list;
	while(this_service!=NULL){
		next_service=this_service->next;
		free(this_service);
		this_service=next_service;
		}

	/* reset the service pointer */
	service_list=NULL;

#ifdef DEBUG1
	printf("\tservice_list freed\n");
#endif

	/* free memory for the command list */
	this_command=command_list;
	while(this_command!=NULL){
		next_command=this_command->next;
		free(this_command);
		this_command=next_command;
	        }

	/* reset the command list */
	command_list=NULL;

#ifdef DEBUG1
	printf("\tcommand_list freed\n");
#endif

	/* free memory for the high priority event list */
	this_event=event_list_high;
	while(this_event!=NULL){
		next_event=this_event->next;
		free(this_event);
		this_event=next_event;
	        }

	/* reset the event pointer */
	event_list_high=NULL;

	/* free memory for the low priority event list */
	this_event=event_list_low;
	while(this_event!=NULL){
		next_event=this_event->next;
		free(this_event);
		this_event=next_event;
	        }

	/* reset the event pointer */
	event_list_low=NULL;

#ifdef DEBUG1
	printf("\tevent_lists freed\n");
#endif

	/* free memory used by my_strtok() function */
	if(original_my_strtok_buffer!=NULL)
		free(original_my_strtok_buffer);

	/* reset the my_strtok() buffers */
	original_my_strtok_buffer=NULL;
	my_strtok_buffer=NULL;

#ifdef DEBUG1
	printf("\tmy_strtok() buffers freed\n");
#endif

	/* free any notification list that may have been overlooked */
	free_notification_list();

#ifdef DEBUG1
	printf("\tnotification_list freed\n");
#endif

#ifdef DEBUG0
	printf("free_memory() end\n");
#endif

	return;
	}


/* free a notification list that was created */
void free_notification_list(void){
	notification *temp_notification;
	notification *next_notification;

#ifdef DEBUG0
	printf("free_notification_list() start\n");
#endif

	temp_notification=notification_list;
	while(temp_notification!=NULL){
		next_notification=temp_notification->next;
		free((void *)temp_notification);
		temp_notification=next_notification;
	        }

	/* reset notification list pointer */
	notification_list=NULL;

#ifdef DEBUG0
	printf("free_notification_list() end\n");
#endif

	return;
        }


/* reset all system-wide variables, so when we've receive a SIGHUP we can restart cleanly */
int reset_variables(void){

#ifdef DEBUG0
	printf("reset_variables() start\n");
#endif

	strcpy(hosts_config_file,"");
	strncpy(log_file,DEFAULT_LOG_FILE,sizeof(log_file));
	log_file[sizeof(log_file)-1]='\x0';
	strncpy(temp_file,DEFAULT_TEMP_FILE,sizeof(temp_file));
	temp_file[sizeof(temp_file)-1]='\x0';
	strncpy(status_file,DEFAULT_STATUS_FILE,sizeof(status_file));
	status_file[sizeof(status_file)-1]='\x0';
	strncpy(command_file,DEFAULT_COMMAND_FILE,sizeof(command_file));
	command_file[sizeof(command_file)-1]='\x0';
	strncpy(comment_file,DEFAULT_COMMENT_FILE,sizeof(comment_file));
	comment_file[sizeof(comment_file)-1]='\x0';
	strncpy(log_archive_path,DEFAULT_LOG_ARCHIVE_PATH,sizeof(log_archive_path));
	log_archive_path[sizeof(log_archive_path)-1]='\x0';

	strcpy(generic_summary,"");

	strcpy(global_host_event_handler,"");
	strcpy(global_service_event_handler,"");
	
	my_strtok_buffer=NULL;
	original_my_strtok_buffer=NULL;

	log_level=DEFAULT_LOG_LEVEL;
	use_syslog=DEFAULT_USE_SYSLOG;
	syslog_level=DEFAULT_SYSLOG_LEVEL;
	log_notifications=DEFAULT_NOTIFICATION_LOGGING;
	log_service_retries=DEFAULT_LOG_SERVICE_RETRIES;
	log_host_retries=DEFAULT_LOG_HOST_RETRIES;
	log_event_handlers=DEFAULT_LOG_EVENT_HANDLERS;

	sleep_time=DEFAULT_SLEEP_TIME;
	interval_length=DEFAULT_INTERVAL_LENGTH;
	inter_check_delay=DEFAULT_INTER_CHECK_DELAY;
	inter_check_delay_method=ICD_SMART;
	interleave_factor=DEFAULT_INTERLEAVE_FACTOR;
	interleave_factor_method=ILF_SMART;

	max_child_process_time=DEFAULT_MAX_CHILD_PROCESS_TIME;

	use_agressive_host_checking=DEFAULT_AGRESSIVE_HOST_CHECKING;

	zombie_cleanup_interval=DEFAULT_ZOMBIE_CLEANUP_INTERVAL;
	command_check_interval=DEFAULT_COMMAND_CHECK_INTERVAL;
	service_check_reaper_interval=DEFAULT_SERVICE_REAPER_INTERVAL;

	check_external_commands=DEFAULT_CHECK_EXTERNAL_COMMANDS;

	log_rotation_method=LOG_ROTATION_NONE;

	last_zombie_cleanup=0L;
	last_command_check=0L;
	last_log_rotation=0L;

        max_parallel_service_checks=DEFAULT_MAX_PARALLEL_SERVICE_CHECKS;
        currently_running_service_checks=0;

	contact_list=NULL;
	contactgroup_list=NULL;
	host_list=NULL;
	hostgroup_list=NULL;
	service_list=NULL;
	event_list_high=NULL;
	event_list_low=NULL;
	notification_list=NULL;
	command_list=NULL;
	timeperiod_list=NULL;

	current_comment_id=0;

	program_mode=ACTIVE_MODE;
	last_mode_change=time(NULL);

#ifdef DEBUG0
	printf("reset_variables() end\n");
#endif

	return OK;
        }



/* removes invalid and old comments from the comment file */
int clean_comment_file(void){
	char input_buffer[MAX_INPUT_BUFFER];
	char *temp_buffer;
	FILE *fpin;
	FILE *fpout;
	int save_comment=TRUE;
	host *temp_host;
	service *temp_service;

#ifdef DEBUG0
	printf("clean_comment_file() start\n");
#endif

	fpin=fopen(comment_file,"r");
	if(fpin==NULL)
		return ERROR;

	fpout=fopen(temp_file,"w");
	if(fpout==NULL){
		fclose(fpin);
		return ERROR;
	        }

	/* process each line in the old comment file */
	while(fgets(input_buffer,(int)(sizeof(input_buffer)-1),fpin)){

		if(feof(fpin))
			break;

		save_comment=TRUE;

		/* this is a host comment */
		if(strstr(input_buffer,"] HOST_COMMENT;")){
		
			temp_buffer=my_strtok(input_buffer,";");
			temp_buffer=my_strtok(NULL,";");
			temp_buffer=my_strtok(NULL,";");

			/* see if the host still exists */
			temp_host=find_host((temp_buffer==NULL)?"":temp_buffer,NULL);
			if(temp_host==NULL)
				save_comment=FALSE;
	                }

		/* this is a service comment */
		else if(strstr(input_buffer,"] SERVICE_COMMENT;")){

			temp_buffer=my_strtok(input_buffer,";");
			temp_buffer=my_strtok(NULL,";");
			temp_buffer=my_strtok(NULL,";");

			/* see if the host still exists */
			temp_host=find_host((temp_buffer==NULL)?"":temp_buffer,NULL);
			if(temp_host==NULL)
				save_comment=FALSE;

			temp_buffer=my_strtok(NULL,";");

			/* see if the service still exists */
			if(temp_host!=NULL)
				temp_service=find_service(temp_host->name,(temp_buffer==NULL)?"":temp_buffer,NULL);
			else
				temp_service=NULL;
			if(temp_service==NULL)
				save_comment=FALSE;
		        }
		
		/* see if the comment is persistent */
		temp_buffer=my_strtok(NULL,";");
		if(temp_buffer==NULL)
			save_comment=FALSE;
		else if(atoi(temp_buffer)==0)
			save_comment=FALSE;

		if(save_comment==TRUE)
			fputs(input_buffer,fpout);
	        }

	fclose(fpout);
	fclose(fpin);

	rename(temp_file,comment_file);

#ifdef DEBUG0
	printf("clean_comment_file() end\n");
#endif

	return OK;
        }


