/**************************************************************************
 *
 * CMD.C -  NetSaint Command CGI
 *
 * Copyright (c) 1999-2000 Ethan Galstad (netsaint@linuxbox.com)
 * Last Modified: 12-27-1999
 *
 * License:
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *************************************************************************/

#include "../common/config.h"
#include "../common/locations.h"
#include "../common/common.h"

#include "cgiutils.h"
#include "getcgi.h"
#include "auth.h"

extern char main_config_file[MAX_FILENAME_LENGTH];
extern char command_file[MAX_FILENAME_LENGTH];
extern char comment_file[MAX_FILENAME_LENGTH];

extern char html_table_bg_odd_color[MAX_COLOR_LENGTH];

extern int  netsaint_process_state;

extern int  check_external_commands;

extern int  use_authentication;

extern comment *comment_list;



/* don't commit any commands for processing while in demo mode... */
/*#define DEMO_MODE*/


char host_name[MAX_HOSTNAME_LENGTH]="";
char service_desc[MAX_SERVICEDESC_LENGTH]="";
char comment_author[MAX_USERNAME_LENGTH]="";
char comment_data[MAX_COMMENT_LENGTH]="";
int comment_id=0;
int notification_delay=0;
int schedule_delay=0;
int persistent_comment=FALSE;

int command_type=CMD_NONE;
int command_mode=CMDMODE_REQUEST;

authdata current_authdata;

void show_command_help(int);
void request_command_data(int);
void commit_command_data(int);
int commit_command(int);
int write_command_to_file(char *);
void clean_comment_data(char *);

void document_header();
void document_footer();
int process_cgivars();


int main(void){
	int result=OK;
	int error=FALSE;
	
	error=process_cgivars();

	document_header();


	if(error==TRUE){
		printf("<p><div align=\"center\"><font color=\"red\"><strong>");
		printf("Invalid argument supplied");
		printf("</strong></font></div></p>\n");
	        }

	else{
	
		reset_cgi_vars();

		result=read_cgi_config_file(DEFAULT_CGI_CONFIG_FILE);
		if(result==ERROR){
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open CGI configuration file '%s' for reading!\n",DEFAULT_CGI_CONFIG_FILE);
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
	                }

		result=read_main_config_file(main_config_file);
		if(result==ERROR){
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open main configuration file '%s' for reading!\n",main_config_file);
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
	                }

		result=read_all_host_config_files(READ_HOSTGROUPS|READ_CONTACTGROUPS|READ_SERVICES|READ_HOSTS);
		if(result==ERROR){
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open host configuration file(s) for reading!\n");
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
                        }
	        }


	/* begin top table */
	printf("<p>\n");
	printf("<table border=0 width=100%%>\n");
	printf("<tr>\n");
	printf("<td align=left valign=top>\n");

	/* left column of the first row */
	printf("<table border=1>\n");
	printf("<tr><td bgcolor=%s>\n",html_table_bg_odd_color);
	printf("<font size=+2><b>CGI Command Interface</b></font><br>\n");
	printf("<font size=-2>\n");
	printf("NetSaint Network Monitor - <a href=\"http://www.netsaint.org\" target=\"_new\">www.netsaint.org</a><br>\n");
	get_netsaint_process_info();
	printf("<font color=\"%s\">%s</font><br>\n",(netsaint_process_state==STATE_OK)?"000000":"FF0000",(netsaint_process_state==STATE_OK)?"NetSaint process is running":"NetSaint process may not be running!");
	printf("</font>\n");
	printf("</td></tr>\n");
	printf("</table>\n");
	printf("</td>\n");

	/* right column of top row */
	printf("<td align=left valign=center>\n");
	printf("<font size=+2>");
	printf("<b>");
	printf("</b>");
	printf("</font>");
	printf("</td>\n");


	/* end of top table */
	printf("</tr>\n");
	printf("</table>\n");
	printf("</p>\n");

	/* if no command was specified... */
	if(command_type==CMD_NONE){

		printf("<p>\n");
		printf("<div align=\"center\">\n");
		printf("<font color=\"red\"><strong>Error: No command was specified</strong></font>\n");
		printf("</div>\n");
		printf("</p>\n");
	        }

	/* if this is the first request for a command, present option */
	else if(command_mode==CMDMODE_REQUEST)
		request_command_data(command_type);

	/* the user wants to commit the command */
	else if(command_mode==CMDMODE_COMMIT)
		commit_command_data(command_type);

	document_footer();

	free_memory();

	return OK;
        }



void document_header(void){
	char date_time[48];
	time_t current_time;
	time_t expire_time;

	time(&current_time);
	get_expire_time_string(&current_time,date_time,sizeof(date_time));

	printf("Cache-Control: no-store\n");
	printf("Pragma: no-cache\n");

	time(&current_time);
	get_expire_time_string(&current_time,date_time,sizeof(date_time));
	printf("Last-Modified: %s\n",date_time);

	expire_time=(time_t)0L;
	get_expire_time_string(&expire_time,date_time,sizeof(date_time));
	printf("Expires: %s\n",date_time);

	printf("Content-type: text/html\r\n\r\n");

	printf("<html>\n");
	printf("<head>\n");
	printf("<title>\n");
	printf("CGI Command Interface\n");
	printf("</title>\n");
	printf("</head>\n");
	printf("<body bgcolor=\"%s\">\n",HTML_BACKGROUND_COLOR);

	return;
        }


void document_footer(void){

	printf("</body>\n");
	printf("</html>\n");

	return;
        }


int process_cgivars(void){
	char **variables;
	int error=FALSE;
	int x;

	variables=getcgivars();

	for(x=0;variables[x]!=NULL;x++){

		/* do some basic length checking on the variable identifier to prevent buffer overflows */
		if(strlen(variables[x])>=MAX_INPUT_BUFFER-1){
			x++;
			continue;
		        }

		/* we found the command type */
		else if(!strcmp(variables[x],"cmd_typ")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			command_type=atoi(variables[x]);
		        }

		/* we found the command mode */
		else if(!strcmp(variables[x],"cmd_mod")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			command_mode=atoi(variables[x]);
		        }

		/* we found the comment id */
		else if(!strcmp(variables[x],"com_id")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			comment_id=atoi(variables[x]);
		        }

		/* we found the notification delay */
		else if(!strcmp(variables[x],"not_dly")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			notification_delay=atoi(variables[x]);
		        }

		/* we found the schedule delay */
		else if(!strcmp(variables[x],"sched_dly")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			schedule_delay=atoi(variables[x]);
		        }

		/* we found the comment author */
		else if(!strcmp(variables[x],"com_author")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			/* protect against buffer overflows */
			if(strlen(variables[x])>=MAX_USERNAME_LENGTH-1){
				error=TRUE;
				break;
			        }
			else
				strcpy(comment_author,variables[x]);
			}

		/* we found the comment data */
		else if(!strcmp(variables[x],"com_data")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			/* protect against buffer overflows */
			if(strlen(variables[x])>=MAX_COMMENT_LENGTH-1){
				error=TRUE;
				break;
			        }
			else
				strcpy(comment_data,variables[x]);
			}

		/* we found the host name */
		else if(!strcmp(variables[x],"host")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			/* protect against buffer overflows */
			if(strlen(variables[x])>=MAX_HOSTNAME_LENGTH-1){
				error=TRUE;
				break;
			        }
			else
				strcpy(host_name,variables[x]);
			}

		/* we found the service name */
		else if(!strcmp(variables[x],"service")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			/* protect against buffer overflows */
			if(strlen(variables[x])>=MAX_SERVICEDESC_LENGTH-1){
				error=TRUE;
				break;
			        }
			else
				strcpy(service_desc,variables[x]);
			}

		/* we got the persistence option for a comment */
		else if(!strcmp(variables[x],"persistent"))
			persistent_comment=TRUE;

	        }

	return error;
        }



void request_command_data(int cmd){

	printf("<p>\n");
	printf("<div align=\"center\">\n");
	printf("<strong>\n");
	printf("You are requesting to ");
	printf("<font color=\"red\">");

	switch(cmd){

	case CMD_ADD_HOST_COMMENT:
		printf("add a host comment");
		break;
		
	case CMD_ADD_SVC_COMMENT:
		printf("add a service comment");
		break;

	case CMD_DEL_HOST_COMMENT:
		printf("delete a host comment");
		break;
		
	case CMD_DEL_SVC_COMMENT:
		printf("delete a service comment");
		break;
		
	case CMD_DELAY_HOST_NOTIFICATION:
		printf("delay a host notification");
		break;

	case CMD_DELAY_SVC_NOTIFICATION:
		printf("delay a service notification");
		break;

	case CMD_DELAY_SVC_CHECK:
		printf("delay a service check");
		break;

	case CMD_IMMEDIATE_SVC_CHECK:
		printf("schedule an immediate service check");
		break;

	case CMD_ENABLE_SVC_CHECK:
		printf("enable a service check");
		break;
		
	case CMD_DISABLE_SVC_CHECK:
		printf("disable a service check");
		break;
		
	case CMD_ENTER_STANDBY_MODE:
		printf("enter STANDBY mode");
		break;
		
	case CMD_ENTER_ACTIVE_MODE:
		printf("enter ACTIVE mode");
		break;
		
	case CMD_SHUTDOWN_PROCESS:
		printf("shutdown the NetSaint process");
		break;

	case CMD_RESTART_PROCESS:
		printf("restart the NetSaint process");
		break;

	case CMD_ENABLE_HOST_SVC_CHECKS:
		printf("enable all service checks for a host");
		break;
		
	case CMD_DISABLE_HOST_SVC_CHECKS:
		printf("disable all service checks for a host");
		break;

	case CMD_DELAY_HOST_SVC_CHECKS:
		printf("delay all service checks for a host");
		break;

	case CMD_IMMEDIATE_HOST_SVC_CHECKS:
		printf("schedule an immediate check of all services for a host");
		break;

	case CMD_DEL_ALL_HOST_COMMENTS:
		printf("delete all comments for a host");
		break;

	case CMD_DEL_ALL_SVC_COMMENTS:
		printf("delete all comments for a service");
		break;

	case CMD_ENABLE_SVC_NOTIFICATIONS:
		printf("enable notifications for a service");
		break;

	case CMD_DISABLE_SVC_NOTIFICATIONS:
		printf("temporarily disable notifications for a service");
		break;

	case CMD_ENABLE_HOST_NOTIFICATIONS:
		printf("enable notifications for a host");
		break;

	case CMD_DISABLE_HOST_NOTIFICATIONS:
		printf("temporarily disable notifications for a host");
		break;

	case CMD_ENABLE_ALL_NOTIFICATIONS_BEYOND_HOST:
		printf("enable notifications for all hosts and services beyond a host");
		break;

	case CMD_DISABLE_ALL_NOTIFICATIONS_BEYOND_HOST:
		printf("temporarily disable notifications for all hosts and services beyond a host");
		break;
		
	case CMD_ENABLE_HOST_SVC_NOTIFICATIONS:
		printf("enable notifications for all services on a host");
		break;

	case CMD_DISABLE_HOST_SVC_NOTIFICATIONS:
		printf("temporarily disable notifications for all services on a host");
		break;

	default:
		printf("execute an unknown command.  Shame on you!");
		printf("</div>\n");
		printf("</p>\n");
		return;
	        }

	printf("</font>\n");
	printf("</strong>\n");
	printf("</div>\n");
	printf("</p>\n");

	printf("<p>\n");
	printf("<div align=\"center\">\n");

	printf("<table border=0>\n");
	printf("<tr>\n");
	printf("<td align=center valign=top>\n");

	printf("<font size=-1><b>Required Information</b></font>\n");
	printf("<table border=1>\n");
	printf("<tr>\n");
	printf("<td>\n");

	printf("<table border=0>\n");

	printf("<tr><td><form method=\"GET\" action=\"%s\"></td><td><INPUT TYPE=\"HIDDEN\" NAME=\"cmd_typ\" VALUE=\"%d\"><INPUT TYPE=\"HIDDEN\" NAME=\"cmd_mod\" VALUE=\"%d\"</td></tr>\n",COMMAND_CGI,cmd,CMDMODE_COMMIT);

	switch(cmd){

	case CMD_ADD_HOST_COMMENT:
		printf("<tr><td><font color=\"red\">Host Name:</font></td><td><b>");
		printf("<INPUT TYPE=\"TEXT\" NAME=\"host\" VALUE=\"%s\">",host_name);
		printf("</b></td></tr>\n");
		printf("<tr><td>Persistent:</td><td><b>");
		printf("<INPUT TYPE=\"checkbox\" NAME=\"persistent\">");
		printf("<tr><td><font color=\"red\">Author (Your Name):</font></td><td><b>");
		printf("<INPUT TYPE=\"TEXT\" NAME=\"com_author\" VALUE=\"%s\">",comment_author);
		printf("<tr><td><font color=\"red\">Comment:</font></td><td><b>");
		printf("<INPUT TYPE=\"TEXT\" NAME=\"com_data\" VALUE=\"%s\" SIZE=40>",comment_data);
		printf("</b></td></tr>\n");
		break;
		
	case CMD_ADD_SVC_COMMENT:
		printf("<tr><td><font color=\"red\">Host Name:</font></td><td><b>");
		printf("<INPUT TYPE=\"TEXT\" NAME=\"host\" VALUE=\"%s\">",host_name);
		printf("</b></td></tr>\n");
		printf("<tr><td><font color=\"red\">Service:</font></td><td><b>");
		printf("<INPUT TYPE=\"TEXT\" NAME=\"service\" VALUE=\"%s\">",service_desc);
		printf("<tr><td>Persistent:</td><td><b>");
		printf("<INPUT TYPE=\"checkbox\" NAME=\"persistent\">");
		printf("<tr><td><font color=\"red\">Author (Your Name):</font></td><td><b>");
		printf("<INPUT TYPE=\"TEXT\" NAME=\"com_author\" VALUE=\"%s\">",comment_author);
		printf("<tr><td><font color=\"red\">Comment:</font></td><td><b>");
		printf("<INPUT TYPE=\"TEXT\" NAME=\"com_data\" VALUE=\"%s\" SIZE=40>",comment_data);
		printf("</b></td></tr>\n");
		break;

	case CMD_DEL_HOST_COMMENT:
	case CMD_DEL_SVC_COMMENT:
		printf("<tr><td><font color=\"red\">Comment ID:</font></td><td><b>");
		printf("<INPUT TYPE=\"TEXT\" NAME=\"com_id\" VALUE=\"%d\">",comment_id);
		printf("</b></td></tr>\n");
		break;
		
	case CMD_DELAY_HOST_NOTIFICATION:
		printf("<tr><td><font color=\"red\">Host Name:</font></td><td><b>");
		printf("<INPUT TYPE=\"TEXT\" NAME=\"host\" VALUE=\"%s\">",host_name);
		printf("</b></td></tr>\n");
		printf("<tr><td><font color=\"red\">Notification Delay (minutes from now):</font></td><td><b>");
		printf("<INPUT TYPE=\"TEXT\" NAME=\"not_dly\" VALUE=\"%d\">",notification_delay);
		printf("</b></td></tr>\n");
		break;

	case CMD_DELAY_SVC_NOTIFICATION:
		printf("<tr><td><font color=\"red\">Host Name:</font></td><td><b>");
		printf("<INPUT TYPE=\"TEXT\" NAME=\"host\" VALUE=\"%s\">",host_name);
		printf("</b></td></tr>\n");
		printf("<tr><td><font color=\"red\">Service:</font></td><td><b>");
		printf("<INPUT TYPE=\"TEXT\" NAME=\"service\" VALUE=\"%s\">",service_desc);
		printf("<tr><td><font color=\"red\">Notification Delay (minutes from now):</font></td><td><b>");
		printf("<INPUT TYPE=\"TEXT\" NAME=\"not_dly\" VALUE=\"%d\">",notification_delay);
		printf("</b></td></tr>\n");
		break;

	case CMD_DELAY_SVC_CHECK:
		printf("<tr><td><font color=\"red\">Host Name:</font></td><td><b>");
		printf("<INPUT TYPE=\"TEXT\" NAME=\"host\" VALUE=\"%s\">",host_name);
		printf("</b></td></tr>\n");
		printf("<tr><td><font color=\"red\">Service:</font></td><td><b>");
		printf("<INPUT TYPE=\"TEXT\" NAME=\"service\" VALUE=\"%s\">",service_desc);
		printf("<tr><td><font color=\"red\">Check Delay (minutes from now):</font></td><td><b>");
		printf("<INPUT TYPE=\"TEXT\" NAME=\"sched_dly\" VALUE=\"%d\">",schedule_delay);
		printf("</b></td></tr>\n");
		break;

	case CMD_DELAY_HOST_SVC_CHECKS:
		printf("<tr><td><font color=\"red\">Host Name:</font></td><td><b>");
		printf("<INPUT TYPE=\"TEXT\" NAME=\"host\" VALUE=\"%s\">",host_name);
		printf("</b></td></tr>\n");
		printf("<tr><td><font color=\"red\">Check Delay (minutes from now):</font></td><td><b>");
		printf("<INPUT TYPE=\"TEXT\" NAME=\"sched_dly\" VALUE=\"%d\">",schedule_delay);
		printf("</b></td></tr>\n");
		break;

	case CMD_IMMEDIATE_SVC_CHECK:
	case CMD_ENABLE_SVC_CHECK:
	case CMD_DISABLE_SVC_CHECK:
	case CMD_DEL_ALL_SVC_COMMENTS:
	case CMD_ENABLE_SVC_NOTIFICATIONS:
	case CMD_DISABLE_SVC_NOTIFICATIONS:
		printf("<tr><td><font color=\"red\">Host Name:</font></td><td><b>");
		printf("<INPUT TYPE=\"TEXT\" NAME=\"host\" VALUE=\"%s\">",host_name);
		printf("</b></td></tr>\n");
		printf("<tr><td><font color=\"red\">Service:</font></td><td><b>");
		printf("<INPUT TYPE=\"TEXT\" NAME=\"service\" VALUE=\"%s\">",service_desc);
		printf("</b></td></tr>\n");
		break;
		
	case CMD_ENABLE_HOST_SVC_CHECKS:
	case CMD_DISABLE_HOST_SVC_CHECKS:
	case CMD_DEL_ALL_HOST_COMMENTS:
	case CMD_IMMEDIATE_HOST_SVC_CHECKS:
	case CMD_ENABLE_HOST_NOTIFICATIONS:
	case CMD_DISABLE_HOST_NOTIFICATIONS:
	case CMD_ENABLE_ALL_NOTIFICATIONS_BEYOND_HOST:
	case CMD_DISABLE_ALL_NOTIFICATIONS_BEYOND_HOST:
	case CMD_ENABLE_HOST_SVC_NOTIFICATIONS:
	case CMD_DISABLE_HOST_SVC_NOTIFICATIONS:
		printf("<tr><td><font color=\"red\">Host Name:</font></td><td><b>");
		printf("<INPUT TYPE=\"TEXT\" NAME=\"host\" VALUE=\"%s\">",host_name);
		printf("</b></td></tr>\n");
		break;
		
	case CMD_ENTER_STANDBY_MODE:
	case CMD_ENTER_ACTIVE_MODE:
	case CMD_SHUTDOWN_PROCESS:
	case CMD_RESTART_PROCESS:
		printf("<tr><td><font color=\"red\">Execution Delay (minutes from now):</font></td><td><b>");
		printf("<INPUT TYPE=\"TEXT\" NAME=\"sched_dly\" VALUE=\"%d\">",schedule_delay);
		printf("</b></td></tr>\n");
		break;
		
	default:
		printf("<tr><td>This should not be happening... :-(</td><td></td></tr>\n");
	        }


	printf("<tr><td></td><td></td></tr>\n");
	printf("<tr><td></td><td><INPUT TYPE=\"Submit\" VALUE=\"Commit\"><INPUT TYPE=\"Reset\" VALUE=\"Reset\"></FORM></td></tr>\n");

	printf("</table>\n");

	printf("</td>\n");
	printf("</tr>\n");
	printf("</table>\n");

	printf("</td>\n");
	printf("<td width=30>&nbsp;</td>\n");
	printf("<td align=center valign=top width=600>\n");

	/* show information about the command... */
	show_command_help(cmd);
	
	printf("</td>\n");
	printf("</tr>\n");
	printf("</table>\n");

	printf("</div>\n");
	printf("</p>\n");

	printf("<p>\n");
	printf("<div align=\"center\">\n");
	printf("<font size=-1>\n");
	printf("Please enter any and all information that may be required before committing the command.<br>Failure to do so will result in an error.");
	printf("</font>\n");
	printf("</div>\n");
	printf("</p>\n");

	return;
        }


void commit_command_data(int cmd){
	int error=TRUE;
	int result=OK;
	int authorized=FALSE;
	service *temp_service;
	host *temp_host;
	comment *temp_comment;

#ifdef DEMO_MODE
	/* demo protection */
	printf("<p>\n");
	printf("<div align=center>\n");
	printf("<b>Sorry, but commands cannot be committed in demo mode!</b>\n");
	printf("</div>\n");
	printf("</p>\n");
	return;
#endif

	/* get authentication information */
	get_authentication_information(&current_authdata);

	switch(cmd){
	case CMD_ADD_HOST_COMMENT:

		/* make sure we have some host name, author name, and comment data... */
		if(strcmp(host_name,"") && strcmp(comment_author,"") && strcmp(comment_data,""))
			error=FALSE;
		else
			error=TRUE;

		/* clean up the comment data */
		clean_comment_data(comment_author);
		clean_comment_data(comment_data);

		/* see if the user is authorized to issue a command... */
		temp_host=find_host(host_name);
		if(is_authorized_for_host_commands(temp_host,&current_authdata)==TRUE)
			authorized=TRUE;
		break;
		
	case CMD_ADD_SVC_COMMENT:
		/* make sure we have some host name and service description, author name, and comment data... */
		if(strcmp(host_name,"") && strcmp(service_desc,"") && strcmp(comment_author,"") && strcmp(comment_data,""))
			error=FALSE;
		else
			error=TRUE;

		/* clean up the comment data */
		clean_comment_data(comment_author);
		clean_comment_data(comment_data);

		/* see if the user is authorized to issue a command... */
		temp_service=find_service(host_name,service_desc);
		if(is_authorized_for_service_commands(temp_service,&current_authdata)==TRUE)
			authorized=TRUE;
		break;

	case CMD_DEL_HOST_COMMENT:
		if(comment_id>0)
			error=FALSE;
		else
			error=TRUE;

		/* read the comment log */
		read_comment_file(comment_file);

		/* find the comment */
		for(temp_comment=comment_list;temp_comment!=NULL;temp_comment=temp_comment->next){
			if(temp_comment->comment_id==comment_id && temp_comment->comment_type==HOST_COMMENT)
				break;
		        }
		if(temp_comment==NULL)
			error=TRUE;

		/* see if the user is authorized to issue a command... */
		if(temp_comment!=NULL){
			temp_host=find_host(temp_comment->host_name);
			if(is_authorized_for_host_commands(temp_host,&current_authdata)==TRUE)
			authorized=TRUE;
		        }
		break;
		
	case CMD_DEL_SVC_COMMENT:
		if(comment_id>0)
			error=FALSE;
		else
			error=TRUE;

		/* read the comment log */
		read_comment_file(comment_file);

		/* find the comment */
		for(temp_comment=comment_list;temp_comment!=NULL;temp_comment=temp_comment->next){
			if(temp_comment->comment_id==comment_id && temp_comment->comment_type==SERVICE_COMMENT)
				break;
		        }
		if(temp_comment==NULL)
			error=TRUE;

		/* see if the user is authorized to issue a command... */
		if(temp_comment!=NULL){
			temp_service=find_service(temp_comment->host_name,temp_comment->service_description);
			if(is_authorized_for_service_commands(temp_service,&current_authdata)==TRUE)
				authorized=TRUE;
		        }
		break;
		
	case CMD_DELAY_HOST_NOTIFICATION:
		/* make sure we have some host name as well as delay time... */
		if(strcmp(host_name,"") && notification_delay>0)
			error=FALSE;
		else
			error=TRUE;

		/* see if the user is authorized to issue a command... */
		temp_host=find_host(host_name);
		if(is_authorized_for_host_commands(temp_host,&current_authdata)==TRUE)
			authorized=TRUE;
		break;

	case CMD_DELAY_SVC_NOTIFICATION:
		/* make sure we have some host name and service description as well as notification delay... */
		if(strcmp(host_name,"") && strcmp(service_desc,"") && notification_delay>0)
			error=FALSE;
		else
			error=TRUE;

		/* see if the user is authorized to issue a command... */
		temp_service=find_service(host_name,service_desc);
		if(is_authorized_for_service_commands(temp_service,&current_authdata)==TRUE)
			authorized=TRUE;
		break;

	case CMD_DELAY_SVC_CHECK:
	case CMD_IMMEDIATE_SVC_CHECK:
	case CMD_ENABLE_SVC_CHECK:
	case CMD_DISABLE_SVC_CHECK:
	case CMD_DEL_ALL_SVC_COMMENTS:
	case CMD_ENABLE_SVC_NOTIFICATIONS:
	case CMD_DISABLE_SVC_NOTIFICATIONS:

		/* make sure we have some host name and service description... */
		if(strcmp(host_name,"") && strcmp(service_desc,""))
			error=FALSE;
		else
			error=TRUE;

		/* see if the user is authorized to issue a command... */
		temp_service=find_service(host_name,service_desc);
		if(is_authorized_for_service_commands(temp_service,&current_authdata)==TRUE)
			authorized=TRUE;
		break;
		
	case CMD_ENTER_STANDBY_MODE:
	case CMD_ENTER_ACTIVE_MODE:
	case CMD_SHUTDOWN_PROCESS:
	case CMD_RESTART_PROCESS:
		error=FALSE;

		/* see if the user is authorized to issue a command... */
		if(is_authorized_for_system_commands(&current_authdata)==TRUE)
			authorized=TRUE;
		break;
		
	case CMD_ENABLE_HOST_SVC_CHECKS:
	case CMD_DISABLE_HOST_SVC_CHECKS:
	case CMD_DEL_ALL_HOST_COMMENTS:
	case CMD_IMMEDIATE_HOST_SVC_CHECKS:
	case CMD_DELAY_HOST_SVC_CHECKS:
	case CMD_ENABLE_HOST_NOTIFICATIONS:
	case CMD_DISABLE_HOST_NOTIFICATIONS:
	case CMD_ENABLE_ALL_NOTIFICATIONS_BEYOND_HOST:
	case CMD_DISABLE_ALL_NOTIFICATIONS_BEYOND_HOST:
	case CMD_ENABLE_HOST_SVC_NOTIFICATIONS:
	case CMD_DISABLE_HOST_SVC_NOTIFICATIONS:

		/* make sure we have some host name... */
		if(strcmp(host_name,""))
			error=FALSE;
		else
			error=TRUE;

		/* see if the user is authorized to issue a command... */
		temp_host=find_host(host_name);
		if(is_authorized_for_host_commands(temp_host,&current_authdata)==TRUE)
			authorized=TRUE;
		break;
		
	default:
		error=TRUE;
	        }


	/* to be safe, we are going to REQUIRE that the authentication functionality is enabled... */
	if(use_authentication==FALSE){
		printf("<p>\n");
		printf("<div align=center>\n");
		printf("<font color=\"red\"><strong>Sorry Dave, I can't let you do that...</strong></font><br><br>");
		printf("It seems that you have chosen to not use the authentication functionality of the CGIs.<br><br>");
		printf("I don't want to be personally responsible for what may happen as a result of allowing unauthorized users to issue commands to NetSaint,");
		printf("so you'll have to disable this safeguard if you are really stubborn and want to invite trouble...<br><br>");
		printf("<strong>Read the section on CGI authentication in the HTML documentation to learn how you can enable authentication and why you should want to.</strong>\n");
		printf("</div>\n");
		printf("</p>\n");
	        }

	/* the user is not authorized to issue the given command */
	else if(authorized==FALSE){
		printf("<p>\n");
		printf("<div align=center>\n");
		printf("<font color=\"red\"><strong>Sorry, but you are not authorized to commit the specified command..</strong></font>");
		printf("</div>\n");
		printf("</p>\n");
	        }

	/* some error occurred (data was probably missing) */
	else if(error==TRUE){
		printf("<p>\n");
		printf("<div align=center>\n");
		printf("<font color=\"red\"><strong>An error occurred while processing your command.<br><br>Go back and verify that you entered all required information correctly...</strong></font>");
		printf("</div>\n");
		printf("</p>\n");
	        }

	/* if NetSaint isn't checking external commands, don't do anything... */
	else if(!check_external_commands){
		printf("<p>\n");
		printf("<div align=center>\n");
		printf("<strong>Sorry, but NetSaint is currently not checking for external commands, so your command will not be committed!</strong>\n");
		printf("</div>\n");
		printf("</p>\n");
	        }
	
	/* everything looks okay, so let's go ahead and commit the command... */
	else{

		/* commit the command */
		result=commit_command(cmd);

		printf("<p>\n");
		printf("<div align=centeR>\n");
		
		if(result==OK){
			printf("Your command request was successfully submitted to NetSaint for processing.<br><br>");
			printf("<font size=-1>Note: It may take a few minutes before the command is actually processed...</font>");
		        }
		else{
			printf("<font color=\"red\"><strong>An error occurred while attempting to commit your command to NetSaint for processing...</strong></font>");
		        }

		printf("</div>\n");
		printf("</p>\n");
	        }

	return;
        }


int commit_command(int cmd){
	char command_buffer[MAX_INPUT_BUFFER];
	time_t current_time;
	time_t scheduled_time;
	time_t notification_time;
	int result;

	/* get the current time */
	time(&current_time);

	/* get the scheduled time */
	scheduled_time=current_time+(schedule_delay*60);

	/* get the notification time */
	notification_time=current_time+(notification_delay*60);

	/* decide how to form the command line... */
	switch(cmd){

	case CMD_ADD_HOST_COMMENT:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] ADD_HOST_COMMENT;%s;%d;%s;%s\n",current_time,host_name,(persistent_comment==TRUE)?1:0,comment_author,comment_data);
		break;
		
	case CMD_ADD_SVC_COMMENT:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] ADD_SVC_COMMENT;%s;%s;%d;%s;%s\n",current_time,host_name,service_desc,(persistent_comment==TRUE)?1:0,comment_author,comment_data);
		break;

	case CMD_DEL_HOST_COMMENT:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] DEL_HOST_COMMENT;%d\n",current_time,comment_id);
		break;
		
	case CMD_DEL_SVC_COMMENT:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] DEL_SVC_COMMENT;%d\n",current_time,comment_id);
		break;
		
	case CMD_DELAY_HOST_NOTIFICATION:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] DELAY_HOST_NOTIFICATION;%s;%lu\n",current_time,host_name,notification_time);
		break;

	case CMD_DELAY_SVC_NOTIFICATION:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] DELAY_SVC_NOTIFICATION;%s;%s;%lu\n",current_time,host_name,service_desc,notification_time);
		break;

	case CMD_DELAY_SVC_CHECK:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] SCHEDULE_SVC_CHECK;%s;%s;%lu\n",current_time,host_name,service_desc,scheduled_time);
		break;

	case CMD_IMMEDIATE_SVC_CHECK:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] SCHEDULE_SVC_CHECK;%s;%s;%lu\n",current_time,host_name,service_desc,current_time);
		break;

	case CMD_ENABLE_SVC_CHECK:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] ENABLE_SVC_CHECK;%s;%s\n",current_time,host_name,service_desc);
		break;
		
	case CMD_DISABLE_SVC_CHECK:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] DISABLE_SVC_CHECK;%s;%s\n",current_time,host_name,service_desc);
		break;
		
	case CMD_ENTER_STANDBY_MODE:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] ENTER_STANDBY_MODE;%lu\n",current_time,scheduled_time);
		break;
		
	case CMD_ENTER_ACTIVE_MODE:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] ENTER_ACTIVE_MODE;%lu\n",current_time,scheduled_time);
		break;
		
	case CMD_SHUTDOWN_PROCESS:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] SHUTDOWN_PROGRAM;%lu\n",current_time,scheduled_time);
		break;

	case CMD_RESTART_PROCESS:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] RESTART_PROGRAM;%lu\n",current_time,scheduled_time);
		break;

	case CMD_ENABLE_HOST_SVC_CHECKS:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] ENABLE_HOST_SVC_CHECKS;%s\n",current_time,host_name);
		break;
		
	case CMD_DISABLE_HOST_SVC_CHECKS:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] DISABLE_HOST_SVC_CHECKS;%s\n",current_time,host_name);
		break;
		
	case CMD_DELAY_HOST_SVC_CHECKS:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] SCHEDULE_HOST_SVC_CHECKS;%s;%lu\n",current_time,host_name,scheduled_time);
		break;

	case CMD_IMMEDIATE_HOST_SVC_CHECKS:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] SCHEDULE_HOST_SVC_CHECKS;%s;%lu\n",current_time,host_name,current_time);
		break;

	case CMD_DEL_ALL_HOST_COMMENTS:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] DEL_ALL_HOST_COMMENTS;%s\n",current_time,host_name);
		break;
		
	case CMD_DEL_ALL_SVC_COMMENTS:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] DEL_ALL_SVC_COMMENTS;%s;%s\n",current_time,host_name,service_desc);
		break;

	case CMD_ENABLE_SVC_NOTIFICATIONS:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] ENABLE_SVC_NOTIFICATIONS;%s;%s\n",current_time,host_name,service_desc);
		break;
		
	case CMD_DISABLE_SVC_NOTIFICATIONS:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] DISABLE_SVC_NOTIFICATIONS;%s;%s\n",current_time,host_name,service_desc);
		break;
		
	case CMD_ENABLE_HOST_NOTIFICATIONS:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] ENABLE_HOST_NOTIFICATIONS;%s\n",current_time,host_name);
		break;
		
	case CMD_DISABLE_HOST_NOTIFICATIONS:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] DISABLE_HOST_NOTIFICATIONS;%s\n",current_time,host_name);
		break;
		
	case CMD_ENABLE_ALL_NOTIFICATIONS_BEYOND_HOST:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] ENABLE_ALL_NOTIFICATIONS_BEYOND_HOST;%s\n",current_time,host_name);
		break;
		
	case CMD_DISABLE_ALL_NOTIFICATIONS_BEYOND_HOST:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] DISABLE_ALL_NOTIFICATIONS_BEYOND_HOST;%s\n",current_time,host_name);
		break;
		
	case CMD_ENABLE_HOST_SVC_NOTIFICATIONS:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] ENABLE_HOST_SVC_NOTIFICATIONS;%s\n",current_time,host_name);
		break;
		
	case CMD_DISABLE_HOST_SVC_NOTIFICATIONS:
		snprintf(command_buffer,sizeof(command_buffer),"[%lu] DISABLE_HOST_SVC_NOTIFICATIONS;%s\n",current_time,host_name);
		break;
		
	default:
		return ERROR;
	        }

	/* make sure command buffer is terminated */
	command_buffer[sizeof(command_buffer)-1]='\x0';

	/* write the command to the command file */
	result=write_command_to_file(command_buffer);

	return result;
        }


/* write a command entry to the command file */
int write_command_to_file(char *cmd){
	FILE *fp;
	mode_t cmode;


	/* open the command for appending, or create it if it doesn't exist */
	fp=fopen(command_file,"a");
	cmode=(mode_t)umask(S_IXUSR|S_IXGRP|S_IXOTH);
	if(fp==NULL){
		printf("<p>\n");
		printf("<div align=center>\n");
		printf("<b>Error: Could not open command file '%s' for update!</b><br><br>\n",command_file);
		printf("Check to make sure that the user running the web server has permission to write to the command file.\n");
		printf("</div>\n");
		printf("</p>\n");
		return ERROR;
	        }

	/* write the command to file */
	fputs(cmd,fp);

	fclose(fp);

	return OK;
        }


/* strips out semicolons from comment data */
void clean_comment_data(char *buffer){
	int x;
	int y;

	y=(int)strlen(buffer);
	
	for(x=0;x<y;x++){
		if(buffer[x]==';')
			buffer[x]=' ';
	        }

	return;
        }


/* display information about a command */
void show_command_help(cmd){

	printf("<font size=-1><b>Command Description</b></font>\n");
	printf("<table border=0 cellpadding=5>\n");
	printf("<tr>\n");
	printf("<td>\n");
	printf("<font size=-1>\n");

	/* decide what information to print out... */
	switch(cmd){

	case CMD_ADD_HOST_COMMENT:
		printf("This command is used to add a comment for the specified host.  If you work with other administrators, you may find it useful to share information about a host\n");
		printf("that is having problems if more than one of you may be working on it.  Unless you check the 'persistent' option, the comment will be automatically be deleted\n");
		printf("the next time NetSaint is restarted.\n");
		break;
		
	case CMD_ADD_SVC_COMMENT:
		printf("This command is used to add a comment for the specified service.  If you work with other administrators, you may find it useful to share information about a host\n");
		printf("or service that is having problems if more than one of you may be working on it.  Unless you check the 'persistent' option, the comment will automatically be\n");
		printf("deleted the next time NetSaint is restarted.\n");
		break;

	case CMD_DEL_HOST_COMMENT:
		printf("This command is used to delete a comment associated with the specified host.\n");
		break;
		
	case CMD_DEL_SVC_COMMENT:
		printf("This command is used to delete a comment associated with the specified service.\n");
		break;
		
	case CMD_DELAY_HOST_NOTIFICATION:
		printf("This command is used to delay the next notification that is sent out for the specified host.  The notification delay will be disregarded if the host changes state\n");
		printf("before the next notification is scheduled to be sent out.  This command has no effect if the host is currently UP.\n");
		break;

	case CMD_DELAY_SVC_NOTIFICATION:
		printf("This command is used to delay the next notification that is sent out for the specified service.  The notification delay will be disregarded if the service changes state\n");
		printf("before the next notification is scheduled to be sent out.  This command has no effect if the service is currently in an OK state.\n");
		break;

	case CMD_DELAY_SVC_CHECK:
		printf("This command is used to delay the next scheduled check of the specified service.  NetSaint will re-queue the service and will not not check it again until the number\n");
		printf("of minutes you specified with the <b><i>check delay</i></b> option has elapsed from the time the command is committed.  Specifying a value of 0 for the delay value\n");
		printf("is equivalent to scheduling an immediate service check.\n");
		break;

	case CMD_IMMEDIATE_SVC_CHECK:
		printf("This command will schedule an immediate check of the specified service.  Note that the check is <i>scheduled</i> immediately, not necessary executed immediately.  If NetSaint\n");
		printf("has fallen behind in its scheduling queue, it will check services that were queued prior to this one.\n");
		break;

	case CMD_ENABLE_SVC_CHECK:
		printf("This command is used to re-enable a service check that has been disabled.  Once a service check is enabled, NetSaint will begin to monitor the service as usual.  If the service\n");
		printf("has recovered from a problem that was detected before the check was disabled, contacts might be notified of the recovery.\n");
		break;
		
	case CMD_DISABLE_SVC_CHECK:
		printf("This command is used to disable a service check.  When a service is disabled NetSaint will not monitor the service.  Doing this will prevent any notifications being sent out for\n");
		printf("the specified service while it is disabled.  In order to have NetSaint check the service in the future you will have to re-enable the service.\n");
		printf("Note that disabling service checks may not necessarily prevent notifications from being sent out about the host which those services are associated with.\n");
		break;
		
	case CMD_ENTER_STANDBY_MODE:
		printf("This command is used to put NetSaint in standby mode.  By supplying a non-zero value for the <b><i>execution delay</i></b> option you can schedule a delayed mode change.\n");
		printf("Specifying a value of 0 for the delay will cause NetSaint to change modes immediately upon processing the request.  In standby mode NetSaint will continue to monitor all hosts\n");
		printf("and services that you have defined, but it will prevent any notifications from being sent out.  Contrast this with active mode, where NetSaint sends out notifications when necessary.\n");
		break;
		
	case CMD_ENTER_ACTIVE_MODE:
		printf("This command is used to put NetSaint in active mode.  By supplying a non-zero value for the <b><i>execution delay</i></b> option you can schedule a delayed mode change.\n");
		printf("Specifying a value of 0 for the delay will cause NetSaint to change modes immediately upon processing the request.  In active mode NetSaint will send out notifications to contacts\n");
		printf("if necessary.  Contrast this with standby mode, where NetSaint will prevent any notifications from being sent to contacts.\n");
		break;
		
	case CMD_SHUTDOWN_PROCESS:
		printf("This command is used to shutdown the NetSaint process.  By supplying a non-zero value for the <b><i>execution delay</i></b> option you can schedule a delayed shutdown.\n");
		printf("Specifying a value of 0 for the delay will cause NetSaint to shut down immediately upon processing the request.  Note: Once the NetSaint has been shutdown, it\n");
		printf("cannot be restarted via the web interface!\n");
		break;

	case CMD_RESTART_PROCESS:
		printf("This command is used to restart the NetSaint process.  By supplying a non-zero value for the <b><i>execution delay</i></b> variable you can schedule a delayed restart.\n");
		printf("Specifying a value of 0 for the delay will cause NetSaint to restart immediately upon processing the request.  Executing a restart command is equivalent to sending\n");
		printf("the process a HUP signal.  All information will be flushed from memory, the configuration files will be re-read, and NetSaint will start monitoring with the new configuration information.\n");
		break;

	case CMD_ENABLE_HOST_SVC_CHECKS:
		printf("This command is used to enable all service checks associated with the specified host.\n");
		break;
		
	case CMD_DISABLE_HOST_SVC_CHECKS:
		printf("This command is used to disable all service checks associated with the specified host.  When a service is disabled NetSaint will not monitor the service.  Doing this will prevent any notifications being sent out for\n");
		printf("the specified service while it is disabled.  In order to have NetSaint check the service in the future you will have to re-enable the service.\n");
		printf("Note that disabling service checks may not necessarily prevent notifications from being sent out about the host which those services are associated with.\n");
		break;
		
	case CMD_DELAY_HOST_SVC_CHECKS:
		printf("This command is used to delay the next scheduled check of all services on the specified host.  NetSaint will re-queue the services and will not not check them again until the number\n");
		printf("of minutes you specified with the <b><i>check delay</i></b> option has elapsed from the time the command is committed.  Specifying a value of 0 for the delay value\n");
		printf("is equivalent to scheduling an immediate check of all the services.\n");
		break;

	case CMD_IMMEDIATE_HOST_SVC_CHECKS:
		printf("This command will schedule an immediate check of all service on the specified host.  Note that the checks are <i>scheduled</i> immediately, not necessary executed immediately.  If NetSaint\n");
		printf("has fallen behind in its scheduling queue, it will check services that were queued prior to these services.\n");
		break;

	case CMD_DEL_ALL_HOST_COMMENTS:
		printf("This command is used to delete all comments associated with the specified host.\n");
		break;
		
	case CMD_DEL_ALL_SVC_COMMENTS:
		printf("This command is used to delete all comments associated with the specified service.\n");
		break;

	case CMD_ENABLE_SVC_NOTIFICATIONS:
		printf("This command is used to enable notifications for the specified service.  Notifications will only be sent out for the\n");
		printf("service state types you defined in your service definition.\n");
		break;

	case CMD_DISABLE_SVC_NOTIFICATIONS:
		printf("This command is used to temporarily prevent notifications from being sent out for the specified service.\n");
		break;

	case CMD_ENABLE_HOST_NOTIFICATIONS:
		printf("This command is used to enable notifications for the specified host.  Notifications will only be sent out for the\n");
		printf("host state types you defined in your host definition.  Note that this command <i>does not</i> enable notifications\n");
		printf("for services associated with this host.\n");
		break;

	case CMD_DISABLE_HOST_NOTIFICATIONS:
		printf("This command is used to temporarily prevent notifications from being sent out for the specified host.  Note that\n");
		printf("this command <i>does not</i> disable notifications for services associated with this host.\n");
		break;

	case CMD_ENABLE_ALL_NOTIFICATIONS_BEYOND_HOST:
		printf("This command is used to enable notifications for all hosts and services that lie \"beyond\" the specified host\n");
		printf("(from the view of NetSaint).\n");
		break;

	case CMD_DISABLE_ALL_NOTIFICATIONS_BEYOND_HOST:
		printf("This command is used to temporarily prevent notifications from being sent out for all hosts and services that lie\n");
		printf("\"beyone\" the specified host (from the view of NetSaint).\n");
		break;
		
	case CMD_ENABLE_HOST_SVC_NOTIFICATIONS:
		printf("This command is used to enable notifications for all services on the specified host.  Notifications will only be sent out for the\n");
		printf("service state types you defined in your service definition.\n");
		break;

	case CMD_DISABLE_HOST_SVC_NOTIFICATIONS:
		printf("This command is used to temporarily prevent notifications from being sent out for all services on the specified host.\n");
		printf("This <i>does not</i> prevent notifications from being sent out about the host.\n");
		break;

	default:
		printf("Sorry, but no information is available for this command.");
	        }

	printf("</font>\n");
	printf("</td>\n");
	printf("</tr>\n");
	printf("</table>\n");

	return;
        }
