/**************************************************************************
 *
 * EXTINFO.C -  NetSaint Extended Information CGI
 *
 * Copyright (c) 1999-2000 Ethan Galstad (netsaint@linuxbox.com)
 * Last Modified: 01-07-2000
 *
 * License:
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *************************************************************************/

#include "../common/config.h"
#include "../common/locations.h"
#include "../common/common.h"

#include "cgiutils.h"
#include "getcgi.h"
#include "auth.h"

extern char            	netsaint_check_command[MAX_INPUT_BUFFER];
extern char		netsaint_process_info[MAX_INPUT_BUFFER];
extern int		netsaint_process_state;
extern int              refresh_rate;

extern time_t		program_start;
extern int		program_mode;
extern time_t           last_mode_change;
extern int              daemon_mode;
extern time_t           last_zombie_cleanup;
extern int              orphaned_child_processes;
extern time_t           last_command_check;
extern time_t           last_log_rotation;

extern char main_config_file[MAX_FILENAME_LENGTH];
extern char url_html_path[MAX_FILENAME_LENGTH];
extern char url_docs_path[MAX_FILENAME_LENGTH];
extern char url_images_path[MAX_FILENAME_LENGTH];
extern char url_logo_images_path[MAX_FILENAME_LENGTH];
extern char url_cgibin_path[MAX_FILENAME_LENGTH];
extern char log_file[MAX_FILENAME_LENGTH];
extern char status_file[MAX_FILENAME_LENGTH];
extern char comment_file[MAX_FILENAME_LENGTH];

extern char html_table_bg_color[MAX_COLOR_LENGTH];
extern char html_table_fg_color[MAX_COLOR_LENGTH];
extern char html_table_bg_odd_color[MAX_COLOR_LENGTH];
extern char html_table_bg_even_color[MAX_COLOR_LENGTH];

extern comment           *comment_list;


#define MAX_MESSAGE_BUFFER		4096

#define HEALTH_WARNING_PERCENTAGE       85
#define HEALTH_CRITICAL_PERCENTAGE      75

void document_header();
void document_footer();
int process_cgivars();

authdata current_authdata;

char host_name[MAX_HOSTNAME_LENGTH]="";
char service_desc[MAX_SERVICEDESC_LENGTH]="";

int display_type=DISPLAY_PROCESS_INFO;

void show_process_info(void);
void show_host_info(void);
void show_service_info(void);


void show_network_health(void);
extern hoststatus *hoststatus_list;
extern servicestatus *servicestatus_list;


int main(void){
	int result=OK;
	int error=FALSE;
	time_t current_time;
	char date_time[48];
	
	error=process_cgivars();


	if(error==TRUE){
		document_header();
		printf("<p><div align=\"center\"><font color=\"red\"><strong>");
		printf("Invalid argument supplied");
		printf("</strong></font></div></p>\n");
		document_footer();
		return ERROR;
	        }

	else{
	
		reset_cgi_vars();

		result=read_cgi_config_file(DEFAULT_CGI_CONFIG_FILE);
		if(result==ERROR){
			document_header();
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open CGI configuration file '%s' for reading!\n",DEFAULT_CGI_CONFIG_FILE);
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
	                }

		result=read_main_config_file(main_config_file);
		if(result==ERROR){
			document_header();
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open main configuration file '%s' for reading!\n",main_config_file);
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
	                }

		result=read_status_log(status_file);
		if(result==ERROR){
			document_header();
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open status log '%s' for reading!\n",status_file);
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
	                }

		result=read_all_host_config_files(READ_HOSTGROUPS|READ_CONTACTGROUPS|READ_HOSTS|READ_SERVICES);
		if(result==ERROR){
			document_header();
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open host configuration file(s) for reading!\n");
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
                        }

	        }


	document_header();

	/* begin top table */
	printf("<p>\n");
	printf("<table border=0>\n");
	printf("<tr>\n");

	/* left column of the first row */
	printf("<td align=left valign=top>\n");

	printf("<table border=1>\n");
	printf("<tr><td bgcolor=%s>\n",html_table_bg_odd_color);
	printf("<font size=+2><b>Extended Information</b></font><br>\n");
	printf("<font size=-2>\n");
	time(&current_time);
	get_time_string(&current_time,date_time,(int)sizeof(date_time));
	printf("Last Updated: %s<BR>",date_time);
	printf("Updated every %d seconds<br>\n",refresh_rate);
	printf("NetSaint Network Monitor - <a href=\"http://www.netsaint.org\" target=\"_new\">www.netsaint.org</a><br>\n");
	get_netsaint_process_info();
	printf("<font color=\"%s\">%s</font><br>\n",(netsaint_process_state==STATE_OK)?"000000":"FF0000",(netsaint_process_state==STATE_OK)?"NetSaint process is running":"NetSaint process may not be running!");
	printf("</font>\n");
	printf("</td></tr>\n");

	if(display_type==DISPLAY_HOST_INFO || display_type==DISPLAY_SERVICE_INFO){
		printf("<tr><td bgcolor=%s>\n",html_table_bg_even_color);
		printf("<font size=-2>\n");
		printf("<a href=\"%s?host=%s\">View history for this host</a><br>\n",HISTORY_CGI,url_encode(host_name));
		printf("<a href=\"%s?host=%s\">View notifications for this</a>\n",NOTIFICATIONS_CGI,url_encode(host_name));
		printf("</font>\n");
		printf("</td></tr>\n");

		printf("<tr><td bgcolor=%s>\n",html_table_bg_odd_color);
		printf("<font size=-2>\n");
		printf("<a href=\"%s?host=%s\">View status detail for this host</a><br>\n",STATUS_CGI,url_encode(host_name));
		if(display_type==DISPLAY_SERVICE_INFO)
			printf("<a href=\"%s?type=%d&host=%s\">View extended information for this host</a>\n",EXTINFO_CGI,DISPLAY_HOST_INFO,url_encode(host_name));
		printf("</font>\n");
		printf("</td></tr>\n");
                }

	printf("</table>\n");

	printf("</td>\n");

	/* center column of top row */
	printf("<td width=30>\n");
	printf("</td>\n");

	/* right column of top row */
	printf("<td align=left valign=center>\n");
	printf("<font size=+2>");
	printf("<b>");
	if(display_type==DISPLAY_HOST_INFO)
		printf("Information For Host '%s'<br clear=all>",host_name);
	else if(display_type==DISPLAY_SERVICE_INFO)
		printf("Information For Service '%s' On Host '%s'",service_desc,host_name);
	else if(display_type==DISPLAY_NETWORK_HEALTH)
		printf("Network Health");
	else
		printf("NetSaint Process Information");
	printf("</b>");
	printf("</font>");
	printf("</td>\n");

	/* end of top table */
	printf("</tr>\n");
	printf("</table>\n");
	printf("</p>\n");

	/* get authentication data */
	get_authentication_information(&current_authdata);

	if(display_type==DISPLAY_HOST_INFO)
		show_host_info();
	else if(display_type==DISPLAY_SERVICE_INFO)
		show_service_info();
	else if(display_type==DISPLAY_NETWORK_HEALTH)
		show_network_health();
	else
		show_process_info();

	document_footer();

	free_memory();

	return OK;
        }



void document_header(void){
	char date_time[48];
	time_t current_time;
	time_t expire_time;


	printf("Cache-Control: no-store\n");
	printf("Pragma: no-cache\n");
	printf("Refresh: %d\n",refresh_rate);

	time(&current_time);
	get_expire_time_string(&current_time,date_time,(int)sizeof(date_time));
	printf("Last-Modified: %s\n",date_time);

	expire_time=(time_t)0L;
	get_expire_time_string(&expire_time,date_time,(int)sizeof(date_time));
	printf("Expires: %s\n",date_time);

	printf("Content-type: text/html\r\n\r\n");

	printf("<html>\n");
	printf("<head>\n");
	printf("<title>\n");
	printf("Extended Information\n");
	printf("</title>\n");
	printf("</head>\n");
	printf("<body bgcolor\"%s\">\n",HTML_BACKGROUND_COLOR);

	return;
        }


void document_footer(void){

	printf("</body>\n");
	printf("</html>\n");

	return;
        }


int process_cgivars(void){
	char **variables;
	int error=FALSE;
	int temp_type;
	int x;

	variables=getcgivars();

	for(x=0;variables[x]!=NULL;x++){

		/* do some basic length checking on the variable identifier to prevent buffer overflows */
		if(strlen(variables[x])>=MAX_INPUT_BUFFER-1){
			x++;
			continue;
		        }

		/* we found the display type */
		else if(!strcmp(variables[x],"type")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
				}
			temp_type=atoi(variables[x]);
			if(temp_type==DISPLAY_HOST_INFO)
				display_type=DISPLAY_HOST_INFO;
			else if(temp_type==DISPLAY_SERVICE_INFO)
				display_type=DISPLAY_SERVICE_INFO;
			else if(temp_type==DISPLAY_NETWORK_HEALTH)
				display_type=DISPLAY_NETWORK_HEALTH;
			else
				display_type=DISPLAY_PROCESS_INFO;
			}

		/* we found the host name */
		else if(!strcmp(variables[x],"host")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			/* protect against buffer overflows */
			if(strlen(variables[x])>=MAX_HOSTNAME_LENGTH-1){
				error=TRUE;
				break;
			        }
			else
				strcpy(host_name,variables[x]);
			}

		/* we found the service name */
		else if(!strcmp(variables[x],"service")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			/* protect against buffer overflows */
			if(strlen(variables[x])>=MAX_SERVICEDESC_LENGTH-1){
				error=TRUE;
				break;
			        }
			else
				strcpy(service_desc,variables[x]);
			}
	        }

	return error;
        }



void show_process_info(void){
	char state_string[MAX_STATUS_LENGTH];
	char state_color[12];
	char date_time[48];
	time_t current_time;
	unsigned long run_time;
	char run_time_string[24];
	int days=0;
	int hours=0;
	int minutes=0;
	int seconds=0;

	/* make sure the user has rights to view system information */
	if(is_authorized_for_system_information(&current_authdata)==FALSE){

		printf("<p>\n");
		printf("<div align=\"center\">\n");
		printf("<strong><font color=red>It appears as though you do not have permission to view process information...</font></strong><br><br>\n");
		printf("If you believe this is an error, check the HTTP server authentication requirements for accessing this CGI<br>");
		printf("and check the authorization options in your CGI configuration file.\n");
		printf("</div>\n");
		printf("</p>\n");

		return;
	        }

	printf("<p>\n");
	printf("<div align=\"center\">\n");

	printf("<strong>Program Information</strong>\n");
	printf("<table border=0>\n");
	printf("<TR>\n");
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Variable</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Value</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("</TR>\n");

	get_time_string(&program_start,date_time,(int)sizeof(date_time));
	printf("<tr bgcolor=\"%s\">",html_table_bg_even_color);
	printf("<td align=left><font size=-1>Program Start Time</font></td>");
	printf("<td><font size=-1>%s</font></td>",date_time);
	printf("</tr>\n");

	time(&current_time);
	run_time=(unsigned long)(current_time-program_start);
	get_time_breakdown(run_time,&days,&hours,&minutes,&seconds);
	sprintf(run_time_string,"%d days %d hrs %d min %d sec",days,hours,minutes,seconds);
	printf("<tr bgcolor=\"%s\">",html_table_bg_odd_color);
	printf("<td align=left><font size=-1>Total Running Time</font></td>");
	printf("<td><font size=-1>%s</font></td>",run_time_string);
	printf("</tr>\n");

	printf("<tr bgcolor=\"%s\">",html_table_bg_even_color);
	printf("<td align=left><font size=-1>Current Program Mode</font></td>");
	printf("<td><table border=0><tr><td bgcolor=\"%s\">&nbsp;%s&nbsp;</td></tr></table></td>",(program_mode==ACTIVE_MODE)?"00FF00":"FFFF00",(program_mode==ACTIVE_MODE)?"ACTIVE":"STANDBY");
	printf("</tr>\n");

	get_time_string(&last_mode_change,date_time,(int)sizeof(date_time));
	printf("<tr bgcolor=\"%s\">",html_table_bg_odd_color);
	printf("<td align=left><font size=-1>Last Mode Change</font></td>");
	printf("<td><font size=-1>%s</font></td>",date_time);
	printf("</tr>\n");

	printf("<tr bgcolor=\"%s\">",html_table_bg_even_color);
	printf("<td align=left><font size=-1>Running As Daemon</font></td>");
	printf("<td><font size=-1>%s</font></td>",(daemon_mode==TRUE)?"Yes":"No");
	printf("</tr>\n");

	printf("<tr bgcolor=\"%s\">",html_table_bg_odd_color);
	printf("<td align=left><font size=-1>Currently Orphaned Child Processes</font></td>");
	printf("<td><table border=0><tr><td bgcolor=\"%s\"><font size=-1>&nbsp;&nbsp;%d&nbsp;&nbsp;</font></td></tr></table></td>",(orphaned_child_processes==0)?"00FF00":"FFFF00",orphaned_child_processes);
	printf("</tr>\n");

	get_time_string(&last_zombie_cleanup,date_time,(int)sizeof(date_time));
	printf("<tr bgcolor=\"%s\">",html_table_bg_even_color);
	printf("<td align=left><font size=-1>Last Zombie Process Cleanup</font></td>");
	printf("<td><font size=-1>%s</font></td>",((unsigned long)last_zombie_cleanup==0L)?"N/A":date_time);
	printf("</tr>\n");

	get_time_string(&last_command_check,date_time,(int)sizeof(date_time));
	printf("<tr bgcolor=\"%s\">",html_table_bg_odd_color);
	printf("<td align=left><font size=-1>Last External Command Check</font></td>");
	printf("<td><font size=-1>%s</font></td>",((unsigned long)last_command_check==0L)?"N/A":date_time);
	printf("</tr>\n");

	get_time_string(&last_log_rotation,date_time,(int)sizeof(date_time));
	printf("<tr bgcolor=\"%s\">",html_table_bg_even_color);
	printf("<td align=left><font size=-1>Last Log File Rotation</font></td>");
	printf("<td><font size=-1>%s</font></td>",((unsigned long)last_log_rotation==0L)?"N/A":date_time);
	printf("</tr>\n");

	printf("</table>\n");

	printf("</div>\n");
	printf("</p>\n");



	printf("<p>\n");
	printf("<div align=\"center\">\n");

	printf("<strong>Process State Information</strong>\n");
	printf("<table border=0>\n");
	printf("<TR>\n");
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Variable</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Value</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("</TR>\n");

	if(netsaint_process_state==STATE_OK){
		strcpy(state_string,"OK");
		strncpy(state_color,OK_COLOR,sizeof(state_color)-1);
		}
	else if(netsaint_process_state==STATE_WARNING){
		strcpy(state_string,"WARNING");
		strncpy(state_color,WARNING_COLOR,sizeof(state_color)-1);
		}
	else if(netsaint_process_state==STATE_CRITICAL){
		strcpy(state_string,"CRITICAL");
		strncpy(state_color,CRITICAL_COLOR,sizeof(state_color)-1);
		}
	else{
		strcpy(state_string,"UNKNOWN");
		strncpy(state_color,UNKNOWN_COLOR,sizeof(state_color)-1);
		}
	printf("<tr bgcolor=\"%s\">",html_table_bg_even_color);
	printf("<td align=left><font size=-1>Process Status</font></td>");
	printf("<td><table border=0><tr><td bgcolor=\"%s\">&nbsp;%s&nbsp;</td></tr></table></td>",state_color,state_string);
	printf("</tr>\n");

	printf("<tr bgcolor=\"%s\">",html_table_bg_odd_color);
	printf("<td align=left><font size=-1>Process Check Command&nbsp;&nbsp;&nbsp;</font></td>");
	printf("<td><font size=-1>%s</font></td>",netsaint_check_command);
	printf("</tr>\n");

	printf("<tr bgcolor=\"%s\">",html_table_bg_even_color);
	printf("<td align=left><font size=-1>Check Command Output&nbsp;&nbsp;&nbsp;</font></td>");
	printf("<td><font size=-1>%s</font></td>",netsaint_process_info);
	printf("</tr>\n");

	printf("</table>\n");

	printf("</div>\n");
	printf("</p>\n");

	printf("<p>\n");
	printf("<div align=\"center\">\n");
	printf("<strong>Process Commands</strong><br>\n");
	printf("<table border=1>\n");
	printf("<tr><td>\n");

	if(netsaint_process_state==STATE_OK){
		printf("<table border=0>\n");

		printf("<tr><td><img src=\"%s%s\" border=0 ALT=\"Shutdown the NetSaint Process\"></td><td valign=center><a href=\"%s?cmd_typ=%d\">Shutdown the NetSaint process</a></td></tr></tr>\n",url_images_path,STOP_ICON,COMMAND_CGI,CMD_SHUTDOWN_PROCESS);
		printf("<tr><td><img src=\"%s%s\" border=0 ALT=\"Restart the NetSaint Process\"></td><td valign=center><a href=\"%s?cmd_typ=%d\">Restart the NetSaint process</a></td></tr></tr>\n",url_images_path,RESTART_ICON,COMMAND_CGI,CMD_RESTART_PROCESS);

		if(program_mode==ACTIVE_MODE)
			printf("<tr><td><img src=\"%s%s\" border=0 ALT=\"Switch To Standby Mode\"></td><td valign=center><a href=\"%s?cmd_typ=%d\">Switch to standby mode</a></td></tr></tr>\n",url_images_path,STANDBY_ICON,COMMAND_CGI,CMD_ENTER_STANDBY_MODE);
		else
			printf("<tr><td><img src=\"%s%s\" border=0 ALT=\"Switch To Active Mode\"></td><td valign=center><a href=\"%s?cmd_typ=%d\">Switch to active mode</a></td></tr></tr>\n",url_images_path,ACTIVE_ICON,COMMAND_CGI,CMD_ENTER_ACTIVE_MODE);

		printf("</table>\n");
	        }
	else{
		printf("<font size=-1 color=\"red\">\n");
		printf("&nbsp;It appears as though NetSaint is not running, so commands are temporarily unavailable...&nbsp;\n");
		printf("</font>\n");
	        }

	printf("</td></tr>\n");
	printf("</table>\n");
	printf("</div>\n");
	printf("</p>\n");

	return;
	}


void show_host_info(void){
	hostextinfo *temp_hostextinfo;
	hoststatus *temp_hoststatus;
	host *temp_host;
	comment *temp_comment;
	int total_comments;
	char date_time[48];
	char state_string[MAX_STATUS_LENGTH];
	char state_color[12];
	unsigned long total_monitored_time;
	unsigned long time_up;
	unsigned long time_down;
	unsigned long time_unreachable;
	float percent_time_up;
	float percent_time_down;
	float percent_time_unreachable;
	char time_up_string[48];
	char time_down_string[48];
	char time_unreachable_string[48];
	char total_time_string[48];
	int days;
	int hours;
	int minutes;
	int seconds;
	int odd=1;


	/* get host info */
	temp_host=find_host(host_name);

	/* make sure the user has rights to view host information */
	if(is_authorized_for_host(temp_host,&current_authdata)==FALSE){

		printf("<p>\n");
		printf("<div align=\"center\">\n");
		printf("<strong><font color=red>It appears as though you do not have permission to view information for this host...</font></strong><br><br>\n");
		printf("If you believe this is an error, check the HTTP server authentication requirements for accessing this CGI<br>");
		printf("and check the authorization options in your CGI configuration file.\n");
		printf("</div>\n");
		printf("</p>\n");

		return;
	        }

	/* get host status info */
	temp_hoststatus=find_hoststatus(host_name);

	/* get extended host info if we can */
	temp_hostextinfo=find_hostextinfo(host_name);

	/* make sure host information exists */
	if(temp_host==NULL){
		printf("<p><div align=\"center\"><font color=\"red\"><strong>Error: Host Not Found!</strong></font></div></p>");
		return;
		}
	if(temp_hoststatus==NULL){
		printf("<p><div align=\"center\"><font color=\"red\"><strong>Error: Host Status Information Not Found!</strong></font></div></p>");
		return;
		}

	printf("<p>\n");
	printf("<div align=\"center\">\n");
	printf("<table border=1>\n");
	printf("<tr><td>");

	printf("<table border=0>\n");
	printf("<tr><td valign=center>");

	if(temp_hostextinfo!=NULL){
		printf("<table border=0 cellspacing=0 cellpadding=0>");
		printf("<tr><td><img src=\"%s%s\" border=0 alt=\"%s\"></td></tr>",url_logo_images_path,temp_hostextinfo->icon_image,temp_hostextinfo->icon_image_alt);
		printf("</table>");
		printf("</td>");
		printf("<td width=10>&nbsp;</td>");
		printf("<td>");
		}	

	printf("<table border=0 cellpadding=0 cellspacing=0>\n");
	printf("<tr><td><b>Host Name:</b></td><td>%s</td></tr>\n",temp_host->name);
	printf("<tr><td><b>Host Description:&nbsp;&nbsp;&nbsp;</b></td><td>%s</td></tr>\n",temp_host->alias);
	printf("<tr><td><b>Host Address:&nbsp;&nbsp;&nbsp;</b></td><td>%s</td></tr>\n",temp_host->address);
	printf("</table>\n");

	printf("</td>\n");
	printf("<td width=10>&nbsp;</td>\n");
	printf("<td>");

	printf("<table border=0 cellpadding=0 cellspacing=15>\n");
	printf("<tr>\n");
	printf("<td><A HREF=\"%s?type=hosts#%s\"><img src=\"%s%s\" border=0 alt=\"View Host Configuration Information\"></A></td>\n",CONFIG_CGI,url_encode(temp_host->name),url_images_path,CONFIGURATION_ICON);
	if(temp_hostextinfo!=NULL && strcmp(temp_hostextinfo->notes_url,""))
		printf("<td><A HREF=\"%s\"><img src=\"%s%s\" border=0 alt=\"View Notes For This Host\"></A></td>\n",temp_hostextinfo->notes_url,url_images_path,NOTES_ICON);
	printf("</tr>\n");
	printf("</table>\n");

	printf("</td></tr>");
	printf("</table>\n");
	printf("</td></tr>\n");
	printf("</table>\n");
	printf("</div>\n");
	printf("</p>\n");


	if(temp_hoststatus->last_check==0L){
		printf("<p>\n");
		printf("<div align=\"center\">\n");
		printf("This host has not yet been checked, so its current status information and state statistics are not available.\n");
		printf("</div>\n");
		printf("</p>\n");
		}
	else{
		printf("<div align=center>\n");
		printf("<table border=0 cellpadding=10>\n");
		printf("<tr>\n");
		printf("<td align=center valign=top>\n");
	
		if(temp_hoststatus->status==HOST_UP){
			strcpy(state_string,"UP");
			strncpy(state_color,HOST_UP_COLOR,sizeof(state_color)-1);
			}
		else if(temp_hoststatus->status==HOST_DOWN){
			strcpy(state_string,"DOWN");
			strncpy(state_color,HOST_DOWN_COLOR,sizeof(state_color)-1);
			}
		else if(temp_hoststatus->status==HOST_UNREACHABLE){
			strcpy(state_string,"UNREACHABLE");
			strncpy(state_color,HOST_UNREACHABLE_COLOR,sizeof(state_color)-1);
			}
		printf("<strong>Host State Information</strong>\n");
		printf("<table border=0>\n");
		printf("<tr bgcolor=\"%s\"><TH align=left><font color=\"%s\">Variable</font></th><TH align=left><font color=\"%s\">Value</font></th></tr>\n",html_table_bg_color,html_table_fg_color,html_table_fg_color);
		printf("<tr bgcolor=\"%s\"><td><font size=-1>Host Status&nbsp;&nbsp;&nbsp;</font></td><td><table border=0><tr><td bgcolor=\"%s\"><font size=-1>&nbsp;%s&nbsp;</font></td></tr></table></td></tr>\n",html_table_bg_even_color,state_color,state_string);
		printf("<tr bgcolor=\"%s\"><td><font size=-1>Status Information&nbsp;&nbsp;&nbsp;</font></td><td><font size=-1>%s&nbsp;</font></td></tr>\n",html_table_bg_odd_color,temp_hoststatus->information);
		get_time_string(&temp_hoststatus->last_check,date_time,(int)sizeof(date_time));
		printf("<tr bgcolor=\"%s\"><td><font size=-1>Last Status Check&nbsp;&nbsp;&nbsp;</font></td><td><font size=-1>%s</font></td></tr>\n",html_table_bg_even_color,date_time);
		get_time_string(&temp_hoststatus->last_state_change,date_time,(int)sizeof(date_time));
		printf("<tr bgcolor=\"%s\"><td><font size=-1>Last State Change&nbsp;&nbsp;&nbsp;</font></td><td><font size=-1>%s</font></td></tr>\n",html_table_bg_odd_color,((unsigned long)temp_hoststatus->last_state_change==0L)?"N/A":date_time);
		get_time_string(&temp_hoststatus->last_notification,date_time,(int)sizeof(date_time));
		printf("<tr bgcolor=\"%s\"><td><font size=-1>Last Host Notification&nbsp;&nbsp;&nbsp;</font></td><td><font size=-1>%s</font></td></tr>\n",html_table_bg_even_color,((unsigned long)temp_hoststatus->last_notification==0L)?"N/A":date_time);
		printf("<tr bgcolor=\"%s\"><td><font size=-1>Host Notifications&nbsp;&nbsp;&nbsp;</font></td><td><table border=0><tr><td bgcolor=\"%s\"><font size=-1>&nbsp;%s&nbsp;</font></td></tr></table></td></tr>\n",html_table_bg_odd_color,(temp_hoststatus->notifications_enabled)?"#00FF00":"#FF0000",(temp_hoststatus->notifications_enabled)?"ENABLED":"DISABLED");
		printf("</table>\n");


		/* calculate host state times */
		calculate_host_state_times(temp_host->name,&total_monitored_time,&time_up,&percent_time_up,&time_down,&percent_time_down,&time_unreachable,&percent_time_unreachable);

		get_time_breakdown(total_monitored_time,&days,&hours,&minutes,&seconds);
		snprintf(total_time_string,sizeof(total_time_string)-1,"%dd %dh %dm %ds",days,hours,minutes,seconds);
		get_time_breakdown(time_up,&days,&hours,&minutes,&seconds);
		snprintf(time_up_string,sizeof(time_up_string)-1,"%dd %dh %dm %ds",days,hours,minutes,seconds);
		get_time_breakdown(time_down,&days,&hours,&minutes,&seconds);
		snprintf(time_down_string,sizeof(time_down_string)-1,"%dd %dh %dm %ds",days,hours,minutes,seconds);
		get_time_breakdown(time_unreachable,&days,&hours,&minutes,&seconds);
		snprintf(time_unreachable_string,sizeof(time_unreachable_string)-1,"%dd %dh %dm %ds",days,hours,minutes,seconds);

		printf("</td>\n");
		printf("<td align=center valign=top>\n");
	
		printf("<strong>Host State Statistics</strong>\n");
		printf("<table border=0>\n");
		printf("<tr bgcolor=\"%s\"><TH align=left><font color=\"%s\">State</font></th><TH align=left><font color=\"%s\">Time</font></th><TH align=left><font color=\"%s\">%% Time</font></th></tr>\n",html_table_bg_color,html_table_fg_color,html_table_fg_color,html_table_fg_color);
		printf("<tr bgcolor=\"%s\"><td bgcolor=%s><font size=-1><strong>UP</strong></font></td><td><font size=-1>%s</font></td><td align=right><font size=-1>%2.1f%%</font></td></tr>\n",html_table_bg_even_color,HOST_UP_COLOR,time_up_string,percent_time_up);
		printf("<tr bgcolor=\"%s\"><td bgcolor=%s><font size=-1><strong>DOWN</strong></font></td><td><font size=-1>%s</font></td><td align=right><font size=-1>%2.1f%%</font></td></tr>\n",html_table_bg_odd_color,HOST_DOWN_COLOR,time_down_string,percent_time_down);
		printf("<tr bgcolor=\"%s\"><td bgcolor=%s><font size=-1><strong>UNREACHABLE</font></strong></td><td><font size=-1>%s</font></td><td align=right><font size=-1>%2.1f%%</font></td></tr>\n",html_table_bg_even_color,HOST_UNREACHABLE_COLOR,time_unreachable_string,percent_time_unreachable);

		printf("<tr><td colspan=3></td></tr>\n");
		printf("<tr bgcolor=\"%s\"><td><font size=-1>All States</font></td><td><font size=-1>%s</font></td><td align=right><font size=-1>100.0%%</font></td></tr>\n",html_table_bg_odd_color,total_time_string);
		printf("</table>\n");

		printf("</td>\n");
		printf("</tr>\n");
		printf("</table>\n");
		printf("</div>\n");
		}

	printf("<p>\n");
	printf("<div align=\"center\">\n");
	printf("<strong>Host Commands</strong><br>\n");
	printf("<table border=1>\n");
	printf("<tr><td>\n");

	if(netsaint_process_state==STATE_OK){
		printf("<table border=0>\n");

		if(temp_hoststatus->notifications_enabled)
			printf("<tr><td><img src=\"%s%s\" border=0 ALT=\"Disable Notifications For This Host\"></td><td valign=center><a href=\"%s?cmd_typ=%d&host=%s\">Disable notifications for this host</a></td></tr></tr>\n",url_images_path,NOTIFICATIONS_DISABLED_ICON,COMMAND_CGI,CMD_DISABLE_HOST_NOTIFICATIONS,url_encode(host_name));
		else
			printf("<tr><td><img src=\"%s%s\" border=0 ALT=\"Enable Notifications For This Host\"></td><td valign=center><a href=\"%s?cmd_typ=%d&host=%s\">Enable notifications for this host</a></td></tr></tr>\n",url_images_path,NOTIFICATION_ICON,COMMAND_CGI,CMD_ENABLE_HOST_NOTIFICATIONS,url_encode(host_name));
		if(temp_hoststatus->status!=HOST_UP)
			printf("<tr><td><img src=\"%s%s\" border=0 ALT=\"Delay Next Host Notification\"></td><td valign=center><a href=\"%s?cmd_typ=%d&host=%s\">Delay next host notification</a></td></tr></tr>\n",url_images_path,DELAY_ICON,COMMAND_CGI,CMD_DELAY_HOST_NOTIFICATION,url_encode(host_name));
		printf("<tr><td><img src=\"%s%s\" border=0 ALT=\"Disable Notifications For All Services On This Host\"></td><td valign=center><a href=\"%s?cmd_typ=%d&host=%s\">Disable notifications for all services on this host</a></td></tr></tr>\n",url_images_path,NOTIFICATIONS_DISABLED_ICON,COMMAND_CGI,CMD_DISABLE_HOST_SVC_NOTIFICATIONS,url_encode(host_name));
		printf("<tr><td><img src=\"%s%s\" border=0 ALT=\"Enable Notifications For All Services On This Host\"></td><td valign=center><a href=\"%s?cmd_typ=%d&host=%s\">Enable notifications for all services on this host</a></td></tr></tr>\n",url_images_path,NOTIFICATION_ICON,COMMAND_CGI,CMD_ENABLE_HOST_SVC_NOTIFICATIONS,url_encode(host_name));
		printf("<tr><td><img src=\"%s%s\" border=0 ALT=\"Schedule An Immediate Check Of All Services On This Host\"></td><td valign=center><a href=\"%s?cmd_typ=%d&host=%s\">Schedule an immediate check of all services on this host</a></td></tr></tr>\n",url_images_path,DELAY_ICON,COMMAND_CGI,CMD_IMMEDIATE_HOST_SVC_CHECKS,url_encode(host_name));
		printf("<tr><td><img src=\"%s%s\" border=0 ALT=\"Disable All Services On This Host\"></td><td valign=center><a href=\"%s?cmd_typ=%d&host=%s\">Disable all services on this host</a></td></tr></tr>\n",url_images_path,DISABLED_ICON,COMMAND_CGI,CMD_DISABLE_HOST_SVC_CHECKS,url_encode(host_name));
		printf("<tr><td><img src=\"%s%s\" border=0 ALT=\"Enable All Services On This Host\"></td><td valign=center><a href=\"%s?cmd_typ=%d&host=%s\">Enable all services on this host</a></td></tr></tr>\n",url_images_path,ENABLED_ICON,COMMAND_CGI,CMD_ENABLE_HOST_SVC_CHECKS,url_encode(host_name));

		printf("</table>\n");
	        }
	else{
		printf("<div align=\"center\"><font size=-1>\n");
		printf("<font color=\"red\">&nbsp;It appears as though NetSaint is not running, so commands are temporarily unavailable...&nbsp;</font><br>\n");
		printf("Click <a href=\"%s?type=%d\">here</a> to view NetSaint process information\n",EXTINFO_CGI,DISPLAY_PROCESS_INFO);
		printf("</font></div>\n");
	        }

	printf("</td></tr>\n");
	printf("</table>\n");
	printf("</div>\n");
	printf("</p>\n");

	printf("<p>\n");
	printf("<div align=\"center\">\n");
	printf("<strong>Host Comments</strong><br>\n");
	printf("<table border=0>\n");
	printf("<tr>\n");
	printf("<td valign=center><img src=%s%s border=0 align=center></td><td valign=center><font size=-1>",url_images_path,COMMENT_ICON);
	printf("<a href=\"%s?cmd_typ=%d&host=%s\">Add a new comment</a>",COMMAND_CGI,CMD_ADD_HOST_COMMENT,url_encode(host_name));
	printf("</font></td>\n");
	printf("</tr>\n");
	printf("<tr>\n");
	printf("<td valign=center><img src=%s%s border=0 align=center></td><td valign=center><font size=-1>",url_images_path,DELETE_ICON);
	printf("<a href=\"%s?cmd_typ=%d&host=%s\">Delete all comments</a>",COMMAND_CGI,CMD_DEL_ALL_HOST_COMMENTS,url_encode(host_name));
	printf("</font></td>\n");
	printf("</tr>\n");
	printf("</table>\n");
	printf("</div>\n");
	printf("</p>\n");


	printf("<p>\n");
	printf("<div align=\"center\">\n");
	printf("<table border=0>\n");
	printf("<tr bgcolor=\"%s\"><td><font color=\"%s\"><strong>Entry Time</strong></font></td><td><font color=\"%s\"><strong>Author</strong></font></td><td><font color=\"%s\"><strong>Comment</strong></font></td><td><font color=\"%s\"><strong>Comment ID</strong></font></td><td><font color=\"%s\"><strong>Actions</strong></font></td></tr>\n",html_table_bg_color,html_table_fg_color,html_table_fg_color,html_table_fg_color,html_table_fg_color,html_table_fg_color);

	/* read in all comments */
	read_comment_file(comment_file);

	for(temp_comment=comment_list;temp_comment!=NULL;temp_comment=temp_comment->next){
		if(temp_comment->comment_type==HOST_COMMENT && !strcmp(temp_comment->host_name,host_name)){
			if(odd)
				odd=0;
			else
				odd=1;
			get_time_string(&temp_comment->entry_time,date_time,(int)sizeof(date_time));
			printf("<tr bgcolor=\"%s\">",(odd)?html_table_bg_odd_color:html_table_bg_even_color);
			printf("<td><font %s size=-1>%s&nbsp;&nbsp;</font></td><td><font size=-1>%s&nbsp;&nbsp;</font></td><td><font size=-1>%s&nbsp;&nbsp;</font></td><td><font size=-1>%d&nbsp;&nbsp;</font></td>",(temp_comment->persistent==TRUE)?"color='red'":"",date_time,temp_comment->author,temp_comment->comment_data,temp_comment->comment_id);
			printf("<td>");
			printf("<a href=\"%s?cmd_typ=%d&com_id=%d\"><img src=%s%s border=0 ALT=\"Delete This Comment\">",COMMAND_CGI,CMD_DEL_HOST_COMMENT,temp_comment->comment_id,url_images_path,DELETE_ICON);
			printf("</td>");
			printf("</tr>\n");
			}
	        }

	printf("</table>\n");

	/* see if this host has any comments associated with it */
	total_comments=number_of_host_comments(temp_host);
	if(total_comments==0)
		printf("This host has no comments associated with it");


	printf("</div>\n");
	printf("</p>\n");
		

	return;
	}


void show_service_info(void){
	int current_status;
	int total_comments;
	comment *temp_comment;
	service *temp_service;
	char date_time[48];
	int odd=0;
	servicestatus *temp_svcstatus;
	char state_string[MAX_STATUS_LENGTH];
	char state_color[12];
	time_t current_time;
	unsigned long time_difference;
	unsigned long total_program_time;
	unsigned long total_monitored_time;
	float percent_time_ok;
	float percent_time_warning;
	float percent_time_unknown;
	float percent_time_critical;
	char time_ok_string[48];
	char time_warning_string[48];
	char time_unknown_string[48];
	char time_critical_string[48];
	char total_time_string[48];
	unsigned long time_ok;
	unsigned long time_warning;
	unsigned long time_unknown;
	unsigned long time_critical;
	int days;
	int hours;
	int minutes;
	int seconds;


	/* find the service */
	temp_service=find_service(host_name,service_desc);

	/* make sure the user has rights to view service information */
	if(is_authorized_for_service(temp_service,&current_authdata)==FALSE){

		printf("<p>\n");
		printf("<div align=\"center\">\n");
		printf("<strong><font color=red>It appears as though you do not have permission to view information for this service...</font></strong><br><br>\n");
		printf("If you believe this is an error, check the HTTP server authentication requirements for accessing this CGI<br>");
		printf("and check the authorization options in your CGI configuration file.\n");
		printf("</div>\n");
		printf("</p>\n");

		return;
	        }

	/* get service status info */
	temp_svcstatus=find_servicestatus(host_name,service_desc);

	/* make sure service information exists */
	if(temp_svcstatus==NULL){
		printf("<p><div align=\"center\"><font color=\"red\"><strong>Error: Service Not Found!</strong></font></div></p>");
		return;
		}

	printf("<p>\n");
	printf("<div align=\"center\">\n");
	printf("<table border=1>\n");
	printf("<tr><td>\n");
	printf("<table border=0>\n");
	printf("<tr><td>\n");
	printf("<table border=0 cellpadding=0 cellspacing=0>\n");
	printf("<tr><td><b>Service Description:&nbsp;&nbsp;&nbsp;</b></td><td>%s</td></tr>\n",temp_service->description);
	printf("<tr><td><b>Host Name:</b></td><td>%s</td></tr>\n",temp_service->host_name);
	printf("</table>\n");

	printf("</td>\n");
	printf("<td width=10>&nbsp;</td>\n");
	printf("<td>");

	printf("<table border=0 cellpadding=0 cellspacing=15>\n");
	printf("<tr>\n");
	printf("<td><A HREF=\"%s?type=services#%s;",CONFIG_CGI,url_encode(temp_service->host_name));
	printf("%s\"><img src=\"%s%s\" border=0 alt=\"View Service Configuration Information\"></A></td>\n",url_encode(temp_service->description),url_images_path,CONFIGURATION_ICON);
	printf("</tr>\n");
	printf("</table>\n");

	printf("</td></tr>\n");
	printf("</table>\n");
	printf("</td></tr>\n");
	printf("</table>\n");
	printf("</div>\n");
	printf("</p>\n");

	if(temp_svcstatus->last_check==0L){
		printf("<p>\n");
		printf("<div align=\"center\">\n");
		printf("This service has not yet been checked, so its current status information and state statistics are not available.\n");
		printf("</div>\n");
		printf("</p>\n");
		}

	else{

		printf("<div align=center>\n");
		printf("<table border=0 cellpadding=10>\n");
		printf("<tr>\n");
		printf("<td align=center valign=top>\n");

		printf("<strong>Service State Information</strong>\n");
		printf("<table border=0>\n");
		printf("<tr bgcolor=\"%s\"><TH align=left><font color=\"%s\">Variable</font></th><TH align=left><font color=\"%s\">Value</font></th></tr>\n",html_table_bg_color,html_table_fg_color,html_table_fg_color);

		if(temp_svcstatus->status==SERVICE_OK || temp_svcstatus->status==SERVICE_RECOVERY){
			strcpy(state_string,"OK");
			strncpy(state_color,OK_COLOR,sizeof(state_color)-1);
			}
		else if(temp_svcstatus->status==SERVICE_WARNING){
			strcpy(state_string,"WARNING");
			strncpy(state_color,WARNING_COLOR,sizeof(state_color)-1);
			}
		else if(temp_svcstatus->status==SERVICE_CRITICAL || temp_svcstatus->status==SERVICE_UNREACHABLE || temp_svcstatus->status==SERVICE_HOST_DOWN){
			strcpy(state_string,"CRITICAL");
			strncpy(state_color,CRITICAL_COLOR,sizeof(state_color)-1);
			}
		else{
			strcpy(state_string,"UNKNOWN");
			strncpy(state_color,UNKNOWN_COLOR,sizeof(state_color)-1);
			}
		printf("<tr bgcolor=\"%s\"><td><font size=-1>Current Status&nbsp;&nbsp;&nbsp;</font></td><td><table border=0><tr><td bgcolor=\"%s\"><font size=-1>&nbsp;%s&nbsp;</font></td></tr></table></td></tr>\n",html_table_bg_even_color,state_color,state_string);

		printf("<tr bgcolor=\"%s\"><td><font size=-1>Status Information&nbsp;&nbsp;&nbsp;</font></td><td><font size=-1>%s&nbsp;</font></td></tr>\n",html_table_bg_odd_color,temp_svcstatus->information);

		printf("<tr bgcolor=\"%s\"><td><font size=-1>Current Attempt&nbsp;&nbsp;&nbsp;</font></td><td><font size=-1>%d/%d&nbsp;</font></td></tr>\n",html_table_bg_even_color,temp_svcstatus->current_attempt,temp_svcstatus->max_attempts);

		printf("<tr bgcolor=\"%s\"><td><font size=-1>State Type&nbsp;&nbsp;&nbsp;</font></td><td><font size=-1>%s&nbsp;</font></td></tr>\n",html_table_bg_odd_color,(temp_svcstatus->state_type==HARD_STATE)?"Hard":"Soft");
		get_time_string(&temp_svcstatus->last_check,date_time,(int)sizeof(date_time));
		printf("<tr bgcolor=\"%s\"><td><font size=-1>Last Status Check&nbsp;&nbsp;&nbsp;</font></td><td><font size=-1>%s</font></td></tr>\n",html_table_bg_even_color,date_time);

		get_time_string(&temp_svcstatus->next_check,date_time,(int)sizeof(date_time));
		printf("<tr bgcolor=\"%s\"><td><font size=-1>Next Scheduled Check&nbsp;&nbsp;&nbsp;</font></td><td><font size=-1>%s</font></td></tr>\n",html_table_bg_odd_color,date_time);

		printf("<tr bgcolor=\"%s\"><td><font size=-1>Service Checks&nbsp;&nbsp;&nbsp;</font></td><td><table border=0><tr><td bgcolor=\"%s\"><font size=-1>&nbsp;%s&nbsp;</font></td></tr></table></td></tr>\n",html_table_bg_even_color,(temp_svcstatus->enabled)?"#00FF00":"#FF0000",(temp_svcstatus->enabled)?"ENABLED":"DISABLED");

		get_time_string(&temp_svcstatus->last_state_change,date_time,(int)sizeof(date_time));
		printf("<tr bgcolor=\"%s\"><td><font size=-1>Last State Change&nbsp;&nbsp;&nbsp;</font></td><td><font size=-1>%s</font></td></tr>\n",html_table_bg_odd_color,(temp_svcstatus->last_state_change==0L)?"N/A":date_time);

		get_time_string(&temp_svcstatus->last_notification,date_time,(int)sizeof(date_time));
		printf("<tr bgcolor=\"%s\"><td><font size=-1>Last Service Notification&nbsp;&nbsp;&nbsp;</font></td><td><font size=-1>%s</font></td></tr>\n",html_table_bg_even_color,(temp_svcstatus->last_notification==0L)?"N/A":date_time);

		printf("<tr bgcolor=\"%s\"><td><font size=-1>Service Notifications&nbsp;&nbsp;&nbsp;</font></td><td><table border=0><tr><td bgcolor=\"%s\"><font size=-1>&nbsp;%s&nbsp;</font></td></tr></table></td></tr>\n",html_table_bg_odd_color,(temp_svcstatus->notifications_enabled)?"#00FF00":"#FF0000",(temp_svcstatus->notifications_enabled)?"ENABLED":"DISABLED");

		printf("</table>\n");
	
		printf("</td>\n");
		printf("<td align=center valign=top>\n");
	
		time(&current_time);

		total_program_time=(unsigned long)(current_time-program_start);

		if((unsigned long)temp_svcstatus->last_state_change==0L)
			time_difference=total_program_time;
		else
			time_difference=(unsigned long)(current_time-temp_svcstatus->last_state_change);


		if(temp_svcstatus->state_type==SOFT_STATE)
			current_status=temp_svcstatus->last_hard_state;
		else
			current_status=temp_svcstatus->status;

		/* calculate service state times */
		calculate_service_state_times(temp_service->host_name,temp_service->description,&total_monitored_time,&time_ok,&percent_time_ok,&time_warning,&percent_time_warning,&time_unknown,&percent_time_unknown,&time_critical,&percent_time_critical);

		get_time_breakdown(time_ok,&days,&hours,&minutes,&seconds);
		snprintf(time_ok_string,sizeof(time_ok_string)-1,"%dd %dh %dm %ds",days,hours,minutes,seconds);
		get_time_breakdown(time_warning,&days,&hours,&minutes,&seconds);
		snprintf(time_warning_string,sizeof(time_warning_string)-1,"%dd %dh %dm %ds",days,hours,minutes,seconds);
		get_time_breakdown(time_unknown,&days,&hours,&minutes,&seconds);
		snprintf(time_unknown_string,sizeof(time_unknown_string)-1,"%dd %dh %dm %ds",days,hours,minutes,seconds);
		get_time_breakdown(time_critical,&days,&hours,&minutes,&seconds);
		snprintf(time_critical_string,sizeof(time_critical_string)-1,"%dd %dh %dm %ds",days,hours,minutes,seconds);

		get_time_breakdown(total_monitored_time,&days,&hours,&minutes,&seconds);
		snprintf(total_time_string,sizeof(total_time_string)-1,"%dd %dh %dm %ds",days,hours,minutes,seconds);

		printf("<strong>Service State Statistics</strong>\n");
		printf("<table border=0>\n");
		printf("<tr bgcolor=\"%s\"><TH align=left><font color=\"%s\">State</font></th><TH align=left><font color=\"%s\">Time</font></th><TH align=left><font color=\"%s\">%% Time</font></th></tr>\n",html_table_bg_color,html_table_fg_color,html_table_fg_color,html_table_fg_color);
		printf("<tr bgcolor=\"%s\"><td bgcolor=%s><font size=-1><strong>OK</strong></font></td><td><font size=-1>%s</font></td><td align=right><font size=-1>%2.1f%%</font></td></tr>\n",html_table_bg_even_color,OK_COLOR,time_ok_string,percent_time_ok);
		printf("<tr bgcolor=\"%s\"><td bgcolor=%s><font size=-1><strong>WARNING</strong></font></td><td><font size=-1>%s</font></td><td align=right><font size=-1>%2.1f%%</font></td></tr>\n",html_table_bg_odd_color,WARNING_COLOR,time_warning_string,percent_time_warning);
		printf("<tr bgcolor=\"%s\"><td bgcolor=%s><font size=-1><strong>UNKNOWN</strong></font></td><td><font size=-1>%s</font></td><td align=right><font size=-1>%2.1f%%</font></td></tr>\n",html_table_bg_even_color,UNKNOWN_COLOR,time_unknown_string,percent_time_unknown);
		printf("<tr bgcolor=\"%s\"><td bgcolor=%s><font size=-1><strong>CRITICAL</strong></font></td><td><font size=-1>%s</font></td><td align=right><font size=-1>%2.1f%%</font></td></tr>\n",html_table_bg_odd_color,CRITICAL_COLOR,time_critical_string,percent_time_critical);

		printf("<tr><td colspan=3></td></tr>\n");
		printf("<tr bgcolor=\"%s\"><td><font size=-1>All States</font></td><td><font size=-1>%s</font></td><td align=right><font size=-1>100.0%%</font></td></tr>\n",html_table_bg_even_color,total_time_string);
		printf("</table>\n");

	
		printf("</td>\n");
		printf("</tr>\n");
		printf("</table>\n");
		printf("</div>\n");
	        }

	printf("<p>\n");
	printf("<div align=\"center\">\n");
	printf("<strong>Service Commands</strong><br>\n");
	printf("<table border=1>\n");
	printf("<tr><td>\n");

	if(netsaint_process_state==STATE_OK){
		printf("<table border=0>\n");

		if(temp_svcstatus->enabled){
			if(temp_svcstatus->notifications_enabled){
				printf("<tr><td><img src=\"%s%s\" border=0 ALT=\"Disable Notifications For This Service\"></td><td valign=center><a href=\"%s?cmd_typ=%d&host=%s",url_images_path,NOTIFICATIONS_DISABLED_ICON,COMMAND_CGI,CMD_DISABLE_SVC_NOTIFICATIONS,url_encode(host_name));
				printf("&service=%s\">Disable notifications for this service</a></td></tr></tr>\n",url_encode(service_desc));
				if(temp_svcstatus->status!=SERVICE_OK){
					printf("<tr><td><img src=\"%s%s\" border=0 ALT=\"Delay Next Service Notification\"></td><td valign=center><a href=\"%s?cmd_typ=%d&host=%s",url_images_path,DELAY_ICON,COMMAND_CGI,CMD_DELAY_SVC_NOTIFICATION,url_encode(host_name));
					printf("&service=%s\">Delay next service notification</a></td></tr></tr>\n",url_encode(service_desc));
			                }
			        }
			else{
				printf("<tr><td><img src=\"%s%s\" border=0 ALT=\"Enable Notifications For This Service\"></td><td valign=center><a href=\"%s?cmd_typ=%d&host=%s",url_images_path,NOTIFICATION_ICON,COMMAND_CGI,CMD_ENABLE_SVC_NOTIFICATIONS,url_encode(host_name));
				printf("&service=%s\">Enable notifications for this service</a></td></tr></tr>\n",url_encode(service_desc));
			        }
			printf("<tr><td><img src=\"%s%s\" border=0 ALT=\"Disable Checks Of This Service\"></td><td valign=center><a href=\"%s?cmd_typ=%d&host=%s",url_images_path,DISABLED_ICON,COMMAND_CGI,CMD_DISABLE_SVC_CHECK,url_encode(host_name));
			printf("&service=%s\">Disable checks of this service</a></td></tr></tr>\n",url_encode(service_desc));
			printf("<tr><td><img src=\"%s%s\" border=0 ALT=\"Delay Next Service Check\"></td><td valign=center><a href=\"%s?cmd_typ=%d&host=%s",url_images_path,DELAY_ICON,COMMAND_CGI,CMD_DELAY_SVC_CHECK,url_encode(host_name));
			printf("&service=%s\">Delay next service check</a></td></tr></tr>\n",url_encode(service_desc));
			printf("<tr><td><img src=\"%s%s\" border=0 ALT=\"Schedule an Immediate Check of this Service\"></td><td valign=center><a href=\"%s?cmd_typ=%d&host=%s",url_images_path,DELAY_ICON,COMMAND_CGI,CMD_IMMEDIATE_SVC_CHECK,url_encode(host_name));
			printf("&service=%s\">Schedule an immediate check of this service</a></td></tr></tr>\n",url_encode(service_desc));
	                }
		else{
			printf("<tr><td><img src=\"%s%s\" border=0 ALT=\"Enable Checks Of This Service\"></td><td valign=center><a href=\"%s?cmd_typ=%d&host=%s",url_images_path,ENABLED_ICON,COMMAND_CGI,CMD_ENABLE_SVC_CHECK,url_encode(host_name));
			printf("&service=%s\">Enable checks of this service</a></td></tr></tr>\n",url_encode(service_desc));
	                }

		printf("</table>\n");
	        }
	else{
		printf("<div align=\"center\"><font size=-1>\n");
		printf("<font color=\"red\">&nbsp;It appears as though NetSaint is not running, so commands are temporarily unavailable...&nbsp;</font><br>\n");
		printf("Click <a href=\"%s?type=%d\">here</a> to view NetSaint process information\n",EXTINFO_CGI,DISPLAY_PROCESS_INFO);
		printf("</font></div>\n");
	        }

	printf("</td></tr>\n");
	printf("</table>\n");
	printf("</div>\n");
	printf("</p>\n");

	printf("<p>\n");
	printf("<div align=\"center\">\n");
	printf("<strong>Service Comments</strong><br>\n");
	printf("<table border=0>\n");
	printf("<tr>\n");
	printf("<td valign=center><img src=%s%s border=0 align=center></td><td valign=center><font size=-1>",url_images_path,COMMENT_ICON);
	printf("<a href=\"%s?cmd_typ=%d&host=%s",COMMAND_CGI,CMD_ADD_SVC_COMMENT,url_encode(host_name));
	printf("&service=%s\">Add a new comment</a>",url_encode(service_desc));
	printf("</font></td>\n");
	printf("</tr>\n");
	printf("<tr>\n");
	printf("<td valign=center><img src=%s%s border=0 align=center></td><td valign=center><font size=-1>",url_images_path,DELETE_ICON);
	printf("<a href=\"%s?cmd_typ=%d&host=%s",COMMAND_CGI,CMD_DEL_ALL_SVC_COMMENTS,url_encode(host_name));
	printf("&service=%s\">Delete all comments</a>",url_encode(service_desc));
	printf("</font></td>\n");
	printf("</tr>\n");
	printf("</table>\n");

	printf("<p>\n");
	printf("<table border=0>\n");
	printf("<tr bgcolor=\"%s\"><td><font color=\"%s\"><strong>Entry Time</strong></font></td><td><font color=\"%s\"><strong>Author</strong></font></td><td><font color=\"%s\"><strong>Comment</strong></font></td><td><font color=\"%s\"><strong>Comment ID</strong></font></td><td><font color=\"%s\"><strong>Actions</strong></font></td></tr>\n",html_table_bg_color,html_table_fg_color,html_table_fg_color,html_table_fg_color,html_table_fg_color,html_table_fg_color);

	/* read in all comments */
	read_comment_file(comment_file);

	/* display all the comments */
	for(temp_comment=comment_list;temp_comment!=NULL;temp_comment=temp_comment->next){
		if(temp_comment->comment_type==SERVICE_COMMENT && !strcmp(temp_comment->host_name,host_name) && !strcmp(temp_comment->service_description,service_desc)){
			if(odd)
				odd=0;
			else
				odd=1;
			get_time_string(&temp_comment->entry_time,date_time,(int)sizeof(date_time));
			printf("<tr bgcolor=\"%s\">",(odd)?html_table_bg_odd_color:html_table_bg_even_color);
			printf("<td><font %s size=-1>%s&nbsp;&nbsp;</font></td><td><font size=-1>%s&nbsp;&nbsp;</font></td><td><font size=-1>%s&nbsp;&nbsp;</font></td><td><font size=-1>%d&nbsp;&nbsp;</font></td>",(temp_comment->persistent==TRUE)?"color='red'":"",date_time,temp_comment->author,temp_comment->comment_data,temp_comment->comment_id);
			printf("<td>");
			printf("<a href=\"%s?cmd_typ=%d&com_id=%d\"><img src=%s%s border=0 ALT=\"Delete This Comment\"></a>",COMMAND_CGI,CMD_DEL_SVC_COMMENT,temp_comment->comment_id,url_images_path,DELETE_ICON);
			printf("</td>");
			printf("</tr>\n");
			}
	        }

	printf("</table>\n");

	/* see if this service has any comments associated with it */
	total_comments=number_of_service_comments(temp_service);
	if(total_comments==0)
		printf("This service has no comments associated with it");

	printf("</p>\n");

	printf("</div>\n");
	printf("</p>\n");
		
	return;
	}


void show_network_health(void){
	hoststatus *temp_hoststatus;
	host *temp_host;
	int total_hosts=0;
	int total_pending_hosts=0;
	int total_host_health=0;
	int host_health_percentage=0;
	servicestatus *temp_servicestatus;
	service *temp_service;
	int total_services=0;
	int total_disabled_services=0;
	int total_pending_services=0;
	int total_service_health=0;
	int service_health_percentage=0;
	char host_health_image[16];
	char service_health_image[16];

	/* check host health  */
	for(temp_hoststatus=hoststatus_list;temp_hoststatus!=NULL;temp_hoststatus=temp_hoststatus->next){

		temp_host=find_host(temp_hoststatus->host_name);

		/* make sure user is authorized to view this host */
		if(is_authorized_for_host(temp_host,&current_authdata)==FALSE)
			continue;

		if(temp_hoststatus->status!=HOST_PENDING)
			total_hosts++;
		else
			total_pending_hosts++;

		if(temp_hoststatus->status==HOST_UP)
			total_host_health++;
		
	        }

	/* calculate host health percentage... */
	if(total_hosts==0)
		host_health_percentage=0;
	else
		host_health_percentage=(total_host_health*100)/total_hosts;


	/* check service health */
	for(temp_servicestatus=servicestatus_list;temp_servicestatus!=NULL;temp_servicestatus=temp_servicestatus->next){

		temp_service=find_service(temp_servicestatus->host_name,temp_servicestatus->description);

		/* make sure user is authorized to view this host */
		if(is_authorized_for_service(temp_service,&current_authdata)==FALSE)
			continue;

		if(!temp_servicestatus->enabled){
			total_disabled_services++;
			continue;
		        }
		else if(temp_servicestatus->status!=SERVICE_PENDING)
			total_services++;
		else
			total_pending_services++;

		if(temp_servicestatus->status==SERVICE_OK)
			total_service_health+=2;
		else if(temp_servicestatus->status==SERVICE_WARNING || temp_servicestatus->status==SERVICE_UNKNOWN)
			total_service_health++;
	        }

	/* calculate service health percentage... */
	if(total_services==0)
		service_health_percentage=0;
	else
		service_health_percentage=(total_service_health*100)/(total_services*2);



	if(host_health_percentage<HEALTH_CRITICAL_PERCENTAGE)
		strncpy(host_health_image,THERM_CRITICAL_IMAGE,sizeof(host_health_image));
	else if(host_health_percentage<HEALTH_WARNING_PERCENTAGE)
		strncpy(host_health_image,THERM_WARNING_IMAGE,sizeof(host_health_image));
	else
		strncpy(host_health_image,THERM_OK_IMAGE,sizeof(host_health_image));
	host_health_image[sizeof(host_health_image)-1]='\x0';

	if(service_health_percentage<HEALTH_CRITICAL_PERCENTAGE)
		strncpy(service_health_image,THERM_CRITICAL_IMAGE,sizeof(service_health_image));
	else if(service_health_percentage<HEALTH_WARNING_PERCENTAGE)
		strncpy(service_health_image,THERM_WARNING_IMAGE,sizeof(service_health_image));
	else
		strncpy(service_health_image,THERM_OK_IMAGE,sizeof(service_health_image));
	service_health_image[sizeof(service_health_image)-1]='\x0';


	printf("<p>\n");
	printf("<div align=\"center\">\n");

	printf("<table border=0 cellpadding=20>\n");
	printf("<tr>\n");
	printf("<td align=center valign=top>\n");
	
	printf("<p>\n");
	printf("<table border=1>\n");
	printf("<tr>\n");
	printf("<td>\n");
	printf("<table border=0>\n");
	printf("<tr>\n");
	printf("<td><img src=\"%s%s\" border=0></td>\n",url_images_path,THERM_SCALE_IMAGE);
	printf("<td valign=bottom><img src=\"%s%s\" border=0 width=25 height=%d alt=\"%d%% Health\"></td>\n",url_images_path,host_health_image,host_health_percentage,host_health_percentage);
	printf("</tr>\n");
	printf("</table>\n");
	printf("</td>\n");
	printf("</tr>\n");
	printf("</table>\n");
	printf("Host Health: <strong>%d%%</strong>\n",host_health_percentage);
	printf("</p>\n");

	printf("<p>\n");
	printf("<table border=0>\n");
	printf("<tr>\n");
	printf("<td>Hosts Checked:</td><td><strong>%d</strong></td>\n",total_hosts);
	printf("</tr>\n");
	printf("<tr>\n");
	printf("<td>Hosts Pending:</td><td><strong>%d</strong></td>\n",total_pending_hosts);
	printf("</tr>\n");
	printf("</table>\n");
	printf("</p>\n");

	printf("</td>\n");
	printf("<td align=center valign=top>\n");

	printf("<p>\n");
	printf("<table border=1>\n");
	printf("<tr>\n");
	printf("<td>\n");
	printf("<table border=0>\n");
	printf("<tr>\n");
	printf("<td><img src=\"%s%s\" border=0></td>\n",url_images_path,THERM_SCALE_IMAGE);
	printf("<td valign=bottom><img src=\"%s%s\" border=0 width=25 height=%d alt=\"%d%% Health\"></td>\n",url_images_path,service_health_image,service_health_percentage,service_health_percentage);
	printf("</tr>\n");
	printf("</table>\n");
	printf("</td>\n");
	printf("</tr>\n");
	printf("</table>\n");
	printf("Service Health: <strong>%d%%</strong>\n",service_health_percentage);
	printf("</p>\n");

	printf("<p>\n");
	printf("<table border=0>\n");
	printf("<tr>\n");
	printf("<td>Services Checked:</td><td><strong>%d</strong></td>\n",total_services);
	printf("</tr>\n");
	printf("<tr>\n");
	printf("<td>Services Disabled:</td><td><strong>%d</strong></td>\n",total_disabled_services);
	printf("</tr>\n");
	printf("<tr>\n");
	printf("<td>Services Pending:</td><td><strong>%d</strong></td>\n",total_pending_services);
	printf("</tr>\n");
	printf("</table>\n");
	printf("</p>\n");

	printf("</td>\n");
	printf("</tr>\n");
	printf("</table>\n");

	printf("</div>\n");
	printf("</p>\n");

	printf("<p>\n");
	printf("<div align=\"center\">\n");
	printf("Network health statistics are computed based upon the number of hosts and services that have been checked.<br>\n");
	printf("Host and services which have not yet been checked or are disabled are not included in the percentages.\n");
	printf("</div>\n");
	printf("</p>\n");

	return;
        }
