/*****************************************************************************
 *
 * STATUSMAP.C - NetSaint Network Status Map CGI
 *
 * Copyright (c) 1999-2000 Ethan Galstad (netsaint@linuxbox.com)
 * Last Modified:   12-20-1999
 *
 * Description:
 *
 * This CGI will dynamically create a map of all hosts that are being
 * monitored on your network.
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "../common/config.h"
#include "../common/locations.h"
#include "../common/common.h"

#include "cgiutils.h"
#include "getcgi.h"
#include "auth.h"

#include <gd.h>			/* Boutell's GD library function */
#include <gdfonts.h>		/* GD library small font definition */

extern char            netsaint_process_info[MAX_INPUT_BUFFER];
extern int             netsaint_process_state;
extern int             refresh_rate;

#define IMAGE_OUTPUT_FILENAME	"statusmap.png"
#define NETSAINT_ICON_FILENAME	"logo.gd2"
#define UNKNOWN_ICON_FILENAME	"unknown.gd2"

extern char main_config_file[MAX_FILENAME_LENGTH];
extern char physical_images_path[MAX_FILENAME_LENGTH];
extern char url_images_path[MAX_FILENAME_LENGTH];
extern char url_cgibin_path[MAX_FILENAME_LENGTH];
extern char log_file[MAX_FILENAME_LENGTH];
extern char status_file[MAX_FILENAME_LENGTH];

extern char html_table_bg_odd_color[MAX_COLOR_LENGTH];
extern char html_table_bg_even_color[MAX_COLOR_LENGTH];

extern hostgroup *hostgroup_list;
extern hoststatus *hoststatus_list;
extern host *host_list;
extern servicestatus *servicestatus_list;

#define DEFAULT_NODE_WIDTH		40
#define DEFAULT_NODE_HEIGHT		65
#define DEFAULT_HORIZONTAL_SPACING	40
#define DEFAULT_VERTICAL_SPACING	75

#define CREATE_HTML	0
#define CREATE_IMAGE	1

int get_host_draw_data(host *);
void draw_host_and_child_links(host *,int,int);
void draw_host(host *,int,int);
void draw_child_link(host *,int,int,int,int,int);
void draw_line(int,int,int,int,int);
int draw_status_map(host *);
void display_map(void);

void document_header();
void document_footer();
int process_cgivars();

char image_input_file[MAX_FILENAME_LENGTH];
char physical_logo_images_path[MAX_FILENAME_LENGTH];

authdata current_authdata;

int top_number_of_immediate_children=0;
int top_max_child_drawing_width=0;
int max_host_depth;

int vertical_spacing=DEFAULT_VERTICAL_SPACING;
int use_vertical_spacing=FALSE;
int horizontal_spacing=DEFAULT_HORIZONTAL_SPACING;
int use_horizontal_spacing=FALSE;
int node_width=DEFAULT_NODE_WIDTH;
int use_node_width=FALSE;
int node_height=DEFAULT_NODE_HEIGHT;
int use_node_height=FALSE;

int image_width=0;
int image_height=0;
int max_image_width=0;
int use_max_width=FALSE;
int max_image_height=0;
int use_max_height=FALSE;
int scale_image=FALSE;
double scaling_factor=1.0L;

char host_name[MAX_HOSTNAME_LENGTH];
int show_all_hosts=TRUE;
int create_type=CREATE_HTML;

gdImagePtr unknown_logo_image=0;
gdImagePtr status_image=0;
int color_white=0;
int color_black=0;
int color_red=0;
int color_green=0;
int color_blue=0;
int color_yellow=0;
int color_orange=0;
int color_grey=0;

FILE *unknown_logo_file;





int main(int argc, char **argv){
	int result;
	int error;
	time_t t;
	char date_time[48];


	error=process_cgivars();

	reset_cgi_vars();

	result=read_cgi_config_file(DEFAULT_CGI_CONFIG_FILE);
	if(result==ERROR){
		if(create_type==CREATE_HTML){
			document_header();
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open CGI config file '%s' for reading\n",DEFAULT_CGI_CONFIG_FILE);
			printf("</strong></font></div></p>\n");
			document_footer();
		        }
		return ERROR;
                }

	result=read_main_config_file(main_config_file);
	if(result==ERROR){
		if(create_type==CREATE_HTML){
			document_header();
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open main config file '%s' for reading\n",main_config_file);
			printf("</strong></font></div></p>\n");
			document_footer();
		        }
		return ERROR;
                }

	result=read_status_log(status_file);
	if(result==ERROR){
		if(create_type==CREATE_HTML){
			document_header();
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open status log '%s' for reading\n",status_file);
			printf("</strong></font></div></p>\n");
			document_footer();
		        }
		return ERROR;
                }

	result=read_all_host_config_files(READ_HOSTS|READ_HOSTGROUPS|READ_CONTACTGROUPS|READ_SERVICES);
	if(result==ERROR){
		if(create_type==CREATE_HTML){
			document_header();
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open host config file(s) for reading\n");
			printf("</strong></font></div></p>\n");
			document_footer();
		        }
		return ERROR;
                }


	document_header();

	if(create_type==CREATE_HTML){

		printf("<table border=0 width=100%%>\n");
		printf("<tr><td align=left>\n");

		printf("<table border=0>\n");
		printf("<tr><td>\n");

		/* left column of the first row */
		printf("<table border=1>\n");

		printf("<tr><td bgcolor=%s>\n",html_table_bg_odd_color);
		printf("<font size=+2><b>Current Network Status</b></font><br>\n");
		printf("<font size=-2>\n");
		time(&t);
		get_time_string(&t,date_time,sizeof(date_time));
		printf("Last Updated: %s<BR>",date_time);
		printf("Updated every %d seconds<br>\n",refresh_rate);
		printf("NetSaint Network Monitor - <a href=\"http://www.netsaint.org\" target=\"_new\">www.netsaint.org</a><br>\n");
		get_netsaint_process_info();
		printf("<font color=\"%s\">%s</font><br>\n",(netsaint_process_state==STATE_OK)?"":CRITICAL_COLOR,(netsaint_process_state==STATE_OK)?"NetSaint process is running...":"Warning: NetSaint process may not be running!");
		printf("</font>\n");
		printf("</td></tr>\n");
	
		printf("<tr><td bgcolor=%s>\n",html_table_bg_even_color);
		printf("<font size=-2>\n");
		printf("<a href=\"%s?host=all\">View status detail for all hosts</a>\n",STATUS_CGI);
		printf("</font>\n");
		printf("</td></tr>\n");

		printf("<tr><td bgcolor=%s>\n",html_table_bg_odd_color);
		printf("<font size=-2>\n");
		printf("<a href=\"%s?hostgroup=all\">View status overview for all hosts</a>\n",STATUS_CGI);
		printf("</font>\n");
		printf("</td></tr>\n");

		printf("</table>\n");

		printf("</td>\n");

		printf("<td width=30></td>\n");

		printf("<td align=left valign=center>\n");

		if(show_all_hosts==TRUE)
			printf("<font size=+2><b>Status Map For All Hosts</b></font>\n");
		else
			printf("<font size=+2><b>Status Map For Host \"%s\" And Children</b></font>\n",host_name);

		printf("</td></tr>\n");
		printf("</table>\n");

		printf("</td>\n");
	
		printf("<td align=right>\n");

		printf("<table border=0>\n");
		printf("<tr><td valign=top>\n");
		printf("<form method=\"GET\" action=\"%s\">\n",STATUSMAP_CGI);
		printf("<input type='hidden' name='host' value='%s'>\n",host_name);

		printf("<table border=0>\n");
		printf("<tr><td align=left>\n");
		printf("<font size=-2>\n");
		printf("Horizontal node spacing:<br>\n");
		printf("</font>\n");
		printf("<font size=-1>\n");
		printf("<select name='hspacing'>\n");
		printf("<option value=%d %s>%d\n",DEFAULT_HORIZONTAL_SPACING,(horizontal_spacing==DEFAULT_HORIZONTAL_SPACING)?"selected":"",DEFAULT_HORIZONTAL_SPACING);
		printf("<option value=10 %s>10\n",(horizontal_spacing==10)?"selected":"");
		printf("<option value=20 %s>20\n",(horizontal_spacing==20)?"selected":"");
		printf("<option value=60 %s>60\n",(horizontal_spacing==60)?"selected":"");
		printf("</select>\n");
		printf("</font>\n");
		printf("</td></tr>\n");
		printf("<tr><td>\n");
		printf("<font size=-2>\n");
		printf("Vertical node spacing:<br>\n");
		printf("</font>\n");
		printf("<font size=-1>\n");
		printf("<select name='vspacing'>\n");
		printf("<option value=%d %s>%d\n",DEFAULT_VERTICAL_SPACING,(vertical_spacing==DEFAULT_VERTICAL_SPACING)?"selected":"",DEFAULT_VERTICAL_SPACING);
		printf("<option value=25 %s>25\n",(vertical_spacing==25)?"selected":"");
		printf("<option value=50 %s>50\n",(vertical_spacing==50)?"selected":"");
		printf("<option value=100 %s>100\n",(vertical_spacing==100)?"selected":"");
		printf("</select>\n");
		printf("</font>\n");
		printf("</td></tr>\n");
		printf("</table>\n");

		printf("</td><td valign=top>\n");

		printf("<table border=0>\n");
		printf("<tr><td align=left>\n");
		printf("<font size=-2>\n");
		printf("Max image width:<br>\n");
		printf("</font>\n");
		printf("<font size=-1>\n");
		printf("<select name='maxwidth'>\n");
		printf("<option value=0 %s>Unlimited\n",(max_image_width==0)?"selected":"");
		printf("<option value=600 %s>600 pixels\n",(max_image_width==600)?"selected":"");
		printf("<option value=800 %s>800 pixels\n",(max_image_width==800)?"selected":"");
		printf("<option value=1000 %s>1000 pixels\n",(max_image_width==1000)?"selected":"");
		printf("<option value=1200 %s>1200 pixels\n",(max_image_width==1200)?"selected":"");
		printf("<option value=1400 %s>1400 pixels\n",(max_image_width==1400)?"selected":"");
		printf("</select>\n");
		printf("</font>\n");
		printf("</td></tr>\n");
		printf("<tr><td>\n");
		printf("<font size=-2>\n");
		printf("Max image height:<br>\n");
		printf("</font>\n");
		printf("<font size=-1>\n");
		printf("<select name='maxheight'>\n");
		printf("<option value=0 %s>Unlimited\n",(max_image_height==0)?"selected":"");
		printf("<option value=300 %s>300 pixels\n",(max_image_height==300)?"selected":"");
		printf("<option value=400 %s>400 pixels\n",(max_image_height==400)?"selected":"");
		printf("<option value=600 %s>600 pixels\n",(max_image_height==600)?"selected":"");
		printf("<option value=800 %s>800 pixels\n",(max_image_height==800)?"selected":"");
		printf("<option value=1000 %s>1000 pixels\n",(max_image_height==1000)?"selected":"");
		printf("</select>\n");
		printf("<input type='submit' value='Update'>\n");
		printf("</font>\n");
		printf("</td></tr>\n");
		printf("</table>\n");

		printf("</form>\n");
		printf("</td></tr>\n");
		printf("</table>\n");

		printf("</td>\n");

		printf("</tr>\n");
		printf("</table>\n");
	        }


	/* get authentication information */
	get_authentication_information(&current_authdata);

	/* display the network map... */
	display_map();

	document_footer();

	free_memory();

	return OK;
        }



void document_header(void){
	char date_time[48];
	time_t current_time;
	time_t expire_time;


	if(create_type==CREATE_HTML){
		printf("Cache-Control: no-store\n");
		printf("Pragma: no-cache\n");
		printf("Refresh: %d\n",refresh_rate);

		time(&current_time);
		get_expire_time_string(&current_time,date_time,sizeof(date_time));
		printf("Last-Modified: %s\n",date_time);

		expire_time=0L;
		get_expire_time_string(&expire_time,date_time,sizeof(date_time));
		printf("Expires: %s\n",date_time);

		printf("Content-Type: text/html\n\n");

		printf("<html>\n");
		printf("<head>\n");
		printf("<title>\n");
		printf("NetSaint Network Status Map\n");
		printf("</title>\n");
		printf("</head>\n");
		printf("<body bgcolor=\"%s\">\n",HTML_BACKGROUND_COLOR);
	        }

	else{
		printf("Cache-Control: no-store\n");
		printf("Pragma: no-cache\n");

		time(&current_time);
		get_expire_time_string(&current_time,date_time,sizeof(date_time));
		printf("Last-Modified: %s\n",date_time);

		expire_time=(time_t)0L;
		get_expire_time_string(&expire_time,date_time,sizeof(date_time));
		printf("Expires: %s\n",date_time);

		printf("Content-Type: image/png\n\n");
	        }

	return;
        }


void document_footer(void){

	if(create_type==CREATE_HTML){
		printf("</body>\n");
		printf("</html>\n");
	        }

	return;
        }



int process_cgivars(void){
	char **variables;
	int error=FALSE;
	int x;

	variables=getcgivars();

	for(x=0;variables[x]!=NULL;x++){

		/* do some basic length checking on the variable identifier to prevent buffer overflows */
		if(strlen(variables[x])>=MAX_INPUT_BUFFER-1){
			x++;
			continue;
		        }


		/* we found the host argument */
		else if(!strcmp(variables[x],"host")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			/* protect against buffer overflows */
			if(strlen(variables[x])>=MAX_HOSTNAME_LENGTH-1)
				strcpy(host_name,"all");
			else{
				strncpy(host_name,variables[x],sizeof(host_name));
				host_name[sizeof(host_name)-1]='\x0';
			        }

			if(!strcmp(host_name,"all"))
				show_all_hosts=TRUE;
			else
				show_all_hosts=FALSE;
		        }

		/* we found the node width argument */
		else if(!strcmp(variables[x],"nodewidth")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			node_width=atoi(variables[x]);
			if(node_width<=0)
				node_width=DEFAULT_NODE_WIDTH;
			use_node_width=TRUE;
		        }

		/* we found the node height argument */
		else if(!strcmp(variables[x],"nodeheight")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			node_height=atoi(variables[x]);
			if(node_height<=0)
				node_height=DEFAULT_NODE_HEIGHT;
			use_node_height=TRUE;
		        }

		/* we found the horizontal spacing argument */
		else if(!strcmp(variables[x],"hspacing")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			horizontal_spacing=atoi(variables[x]);
			if(horizontal_spacing<=0)
				horizontal_spacing=DEFAULT_HORIZONTAL_SPACING;
			use_horizontal_spacing=TRUE;
		        }

		/* we found the vertical spacing argument */
		else if(!strcmp(variables[x],"vspacing")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			vertical_spacing=atoi(variables[x]);
			if(vertical_spacing<=0)
				vertical_spacing=DEFAULT_VERTICAL_SPACING;
			use_vertical_spacing=TRUE;
		        }

		/* we found the max image width argument */
		else if(!strcmp(variables[x],"maxwidth")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			max_image_width=atoi(variables[x]);
			if(max_image_width<=0){
				max_image_width=1;
				use_max_width=FALSE;
			        }
			else{
				scale_image=TRUE;
				use_max_width=TRUE;
			        }
		        }

		/* we found the max image height argument */
		else if(!strcmp(variables[x],"maxheight")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			max_image_height=atoi(variables[x]);
			if(max_image_height<=0){
				max_image_height=1;
				use_max_height=FALSE;
			        }
			else{
				scale_image=TRUE;
				use_max_height=TRUE;
			        }
		        }

		/* we found the image creation option */
		else if(!strcmp(variables[x],"createimage")){
			create_type=CREATE_IMAGE;
		        }
	        }

	return error;
        }



/* top-level map generation... */
void display_map(void){
	host *temp_host=NULL;


	if(create_type==CREATE_IMAGE){

		/* get the path where we will be reading logo images from (GD2 format)... */
		snprintf(physical_logo_images_path,sizeof(physical_logo_images_path),"%slogos/",physical_images_path);
		physical_logo_images_path[sizeof(physical_logo_images_path)-1]='\x0';

		/* load the unknown icon to use for hosts that don't have pretty images associated with them... */
		snprintf(image_input_file,sizeof(image_input_file),"%s%s",physical_logo_images_path,UNKNOWN_ICON_FILENAME);
		image_input_file[sizeof(image_input_file)-1]='\x0';

		unknown_logo_file=fopen(image_input_file,"rb");
		if(unknown_logo_file!=NULL){
			unknown_logo_image=gdImageCreateFromGd2(unknown_logo_file);
			fclose(unknown_logo_file);
	                }
	        }

	if(create_type==CREATE_HTML)
		printf("<map name='statusmap'>\n");

	/* create the network image and print out the imagemap coordinates... */
	if(show_all_hosts==TRUE)
		draw_status_map(NULL);
	else{
		temp_host=find_host(host_name);
		draw_status_map(temp_host);
                }

	if(create_type==CREATE_IMAGE){

		/* free memory allocated to the unknown image */
		if(unknown_logo_image)
			gdImageDestroy(unknown_logo_image);
	        }

	if(create_type==CREATE_HTML){

		printf("</map>\n");

		/* display raw and scaled image sizes - more for debugging information than anything... */
		printf("<p>\n");
		printf("<font size=-1 color=\"#ff0000\">\n");
		printf("Raw Image Size: %dx%d pixels<br>",image_width,image_height);
		printf("Scaled Image Size: %dx%d pixels",(int)((double)image_width*scaling_factor),(int)((double)image_height*scaling_factor));
		printf("</font>\n");
		printf("</p>\n");

		if(show_all_hosts==FALSE){
			
			/* display a link to go "up" the dependency tree... */
			printf("<p>\n");
			printf("<font size=-1>\n");

			printf("<A HREF=\"%s?host=%s",STATUSMAP_CGI,(!strcmp(temp_host->parent_host,""))?"all":url_encode(temp_host->parent_host));

			if(use_node_width==TRUE)
				printf("&nodewidth=%d",node_width);
			if(use_node_height==TRUE)
				printf("&nodeheight=%d",node_height);
			if(use_horizontal_spacing==TRUE)
				printf("&hspacing=%d",horizontal_spacing);
			if(use_vertical_spacing==TRUE)
				printf("&vspacing=%d",vertical_spacing);
			if(use_max_width==TRUE)
				printf("&maxwidth=%d",max_image_width);
			if(use_max_height==TRUE)
				printf("&maxheight=%d",max_image_height);

			printf("\"><IMG SRC=\"%s%s\" BORDER=0 ALT=\"Move up one host dependency level\"></A>\n",url_images_path,PARENT_TRAVERSAL_ICON);

			printf("</font>\n");
			printf("</center>\n");
			printf("</p>\n");
		        }

		/* write the URL location for the image we just generated - the web browser will come and get it... */
		printf("<p>\n");
		printf("<center>\n");
		printf("<img src='%s?host=%s&createimage",STATUSMAP_CGI,url_encode(host_name));
		if(use_node_width==TRUE)
			printf("&nodewidth=%d",node_width);
		if(use_node_height==TRUE)
			printf("&nodeheight=%d",node_height);
		if(use_horizontal_spacing==TRUE)
			printf("&hspacing=%d",horizontal_spacing);
		if(use_vertical_spacing==TRUE)
			printf("&vspacing=%d",vertical_spacing);
		if(use_max_width==TRUE)
			printf("&maxwidth=%d",max_image_width);
		if(use_max_height==TRUE)
			printf("&maxheight=%d",max_image_height);
		printf("' width=%d height=%d border=0 name='statusimage' useMap='#statusmap'>\n",(int)((double)image_width*scaling_factor),(int)((double)image_height*scaling_factor));
		printf("</p>\n");
	        }

	return;
        }



int draw_status_map(host *hst){
	FILE *image_file=NULL;
	int max_child_drawing_width;
	double min_scaling_factor=1.0L;
	double temp_scaling_factor=1.0L;

	if(create_type==CREATE_IMAGE){

		/* use STDOUT for writing the image data... */
		image_file=stdout;
	        }

	/* calculate drawing info */
	get_host_draw_data(hst);

	/* get the top level maximum child drawing width */
	if(hst==NULL)
		max_child_drawing_width=top_max_child_drawing_width;
	else
		max_child_drawing_width=hst->max_child_drawing_width;

	/* calculate the width of the image we have to create */
	image_width=(max_child_drawing_width*node_width)+((max_child_drawing_width-1)*horizontal_spacing)+horizontal_spacing+2;

	/* get the max depth of child hosts beneath the top host */
	max_host_depth=get_max_host_child_depth(hst);

	/* calculate the height of the image we have to create */
	image_height=(max_host_depth*node_height)+((max_host_depth-1)*vertical_spacing)+3;

	/* decide if we need to scale the image */
	if(scale_image==TRUE){

		/* if the user specified a max width and we're over it, compute the scaling factor */
		if(use_max_width && image_width>max_image_width){
			temp_scaling_factor=(double)((double)max_image_width/(double)image_width);
			if(temp_scaling_factor<min_scaling_factor)
				min_scaling_factor=temp_scaling_factor;
		        }

		/* if the user specified a max height and we're over it, compute the scaling factor */
		if(use_max_height==TRUE && image_height>max_image_height){
			temp_scaling_factor=(double)((double)max_image_height/(double)image_height);
			if(temp_scaling_factor<min_scaling_factor)
				min_scaling_factor=temp_scaling_factor;
		        }

		/* check our scaling factor sanity */
		if(min_scaling_factor>=1.0L){
			scale_image=FALSE;
			scaling_factor=1.0L;
		        }
		else
			scaling_factor=min_scaling_factor;
	        }


	if(create_type==CREATE_IMAGE){

		/* allocate buffer for storing image */
		status_image=gdImageCreate(image_width,image_height);
		if(status_image==NULL){
			printf("Error: Could not allocate memory for image\n");
			return ERROR;
	                }

		/* allocate colors used for drawing */
		color_white=gdImageColorAllocate(status_image,255,255,255);
		color_black=gdImageColorAllocate(status_image,0,0,0);
		color_grey=gdImageColorAllocate(status_image,128,128,128);
		color_red=gdImageColorAllocate(status_image,255,0,0);
		color_green=gdImageColorAllocate(status_image,0,175,0);
		color_blue=gdImageColorAllocate(status_image,0,0,255);
		color_yellow=gdImageColorAllocate(status_image,255,255,0);
		color_orange=gdImageColorAllocate(status_image,255,100,25);

		/* set transparency index */
		gdImageColorTransparent(status_image,color_white);

		/* make sure the graphic is interlaced */
		gdImageInterlace(status_image,1);
	        }

	/* start drawing all hosts below and including the "top" host */
	draw_host_and_child_links(hst,image_width/2,1);

	if(create_type==CREATE_IMAGE){

		/* write the image to file */
		gdImagePng(status_image,image_file);

		/* free memory allocated to image */
		gdImageDestroy(status_image);
	        }

	return OK;
        }



/* gets data necessary for drawing the host and its child hosts */
int get_host_draw_data(host *hst){
	int max_width=0;
	int temp_width=0;
	int children=0;
	host *temp_host;

	/* get the number of immediate children to this host */
	children=number_of_immediate_children(hst);

	if(hst==NULL)
		top_number_of_immediate_children=children;
	else{
		hst->number_of_immediate_children=children;
	        }

	/* if host has no children, max drawing width is one unit */
	if(children==0)
		max_width=1;

	/* calculate max width of all children hosts */
	else for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){

		if(is_immediate_child_of_host(hst,temp_host)){

			temp_width=get_host_draw_data(temp_host);
			max_width+=temp_width;
			}
		}

	if(hst==NULL)
		top_max_child_drawing_width=max_width;
	else
		hst->max_child_drawing_width=max_width;

	return max_width;
	}



/* draws a host and any child links */
void draw_host_and_child_links(host *hst,int x,int y){
	host *temp_host;
	int has_children=0;
	int current_child;
	int number_of_immediate_children;
	int max_child_drawing_width;
	int this_draw_width;
	int x1, xx;
	int y0, y1;
	int link_color;
	hoststatus *temp_status;

	/* draw the host (logo, name, and status) */
	draw_host(hst,x,y);

	/* does the host have any children? */
	if(hst==NULL){
		if(top_number_of_immediate_children>0)
			has_children=1;
		else
			has_children=0;
	        }
	else if(hst->number_of_immediate_children>0){
		has_children=1;
	        }
	else{
		has_children=0;
	        }

	/* if the host has children, draw the liks to the child hosts.. */
	if(has_children){

		/* the default child link color is black */
		link_color=color_black;

		/* change the child link color if this host is not up... */
		if(hst!=NULL){

			/* find the status entry for this host */
			temp_status=find_hoststatus(hst->name);

			/* the link to the children should be red if this parent host is either down or unreachable */
			if(temp_status!=NULL){
				if(temp_status->status==HOST_UNREACHABLE || temp_status->status==HOST_DOWN)
					link_color=color_red;
		                 }
		        }

		/* get the number of immediate children and the max child drawing width for this host */
		if(hst==NULL){
			number_of_immediate_children=top_number_of_immediate_children;
			max_child_drawing_width=top_max_child_drawing_width;
		        }
		else{
			number_of_immediate_children=hst->number_of_immediate_children;
			max_child_drawing_width=hst->max_child_drawing_width;
		        }


		/* calculate starting points for drawing... */
		y0=y+node_height;
		y1=y0+vertical_spacing;
		x1=x-(((max_child_drawing_width*node_width)+((max_child_drawing_width-1)*horizontal_spacing))/2);
		xx=x1;

		/* the first potential child is the first host in the host list... */
		temp_host=host_list;

		/* draw the links to all child hosts... */
		for(current_child=1;current_child<=number_of_immediate_children;current_child++){

			/* find the next child host - never go back to the head of the list or we'll have duplicates... */
			for(;temp_host!=NULL;temp_host=temp_host->next){
				if(is_immediate_child_of_host(hst,temp_host))
					break;
				}
			
			/* if for some reason we're at the end of the host list, exit draw loop */
			if(temp_host==NULL)
				break;

			/* calculate the drawing width for this host... */
			this_draw_width=((temp_host->max_child_drawing_width*node_width)+((temp_host->max_child_drawing_width-1)*horizontal_spacing))/2;
			xx+=this_draw_width;

			if(create_type==CREATE_IMAGE){

				/* draw the link to the child host */
				draw_child_link(temp_host,x,y0,xx,y1,link_color);
			        }

			/* draw the child host and its child links */
			draw_host_and_child_links(temp_host,xx,y1);

			/* update the drawing coordinates... */
			xx+=this_draw_width;
			xx+=horizontal_spacing;

			/* go to the next host in the host list - its a potential child... */
			temp_host=temp_host->next;
			}
		}

	return;
	}



/* draws a host */
void draw_host(host *hst,int x,int y){
	int x1=0;
	int x2=0;
	int y2=0;
	int centerx=0;
	int centery=0;
	int color=color_black;
	FILE *logo_file=NULL;
	gdImagePtr logo_image;
	int string_width=0;
	int string_height=0;
	char temp_buffer[MAX_INPUT_BUFFER];
	int status_color;
	int service_totals=0;
	int found_total=0;
	int has_image=0;
	hostextinfo *temp_hostextinfo=NULL;
	hoststatus *temp_status=NULL;
	int authorized_for_host=FALSE;

	/* treat the top level "host" differently and just place the NetSaint logo there */
	if(hst==NULL){

		if(create_type==CREATE_HTML){

			/* get upper left hand corner of bounding box */
			x1=x-(node_width/2);
			x2=x1+node_width;
			y2=y+node_height;

			/* print out coordinates for the imagemap in the HTML code... */
			if(scale_image==TRUE)
				printf("<AREA shape='rect' coords='%d,%d,%d,%d' alt='NetSaint'>\n",(int)((double)x1*scaling_factor),(int)((double)y*scaling_factor),(int)((double)x2*scaling_factor),(int)((double)y2*scaling_factor));
			else
				printf("<AREA shape='rect' coords='%d,%d,%d,%d' alt='NetSaint'>\n",x1,y,x2,y2);
		        }


		if(create_type==CREATE_IMAGE){

			/* get the name of the image file to open for the logo */
			snprintf(image_input_file,sizeof(image_input_file),"%s%s",physical_logo_images_path,NETSAINT_ICON_FILENAME);
			image_input_file[sizeof(image_input_file)-1]='\x0';

			/* read in the logo... */
			logo_file=fopen(image_input_file,"rb");
			if(logo_file!=NULL){

				logo_image=gdImageCreateFromGd2(logo_file);
				fclose(logo_file);

				/* if we were able to load the image... */
				if(logo_image!=NULL){

					/* get upper left hand corner of bounding box (image dimensions may be different than expected... ) */
					x1=x-(logo_image->sx/2);
					x2=x1+logo_image->sx;
					y2=y+logo_image->sy;

				        /* copy the logo image to the main image we are creating... */
					gdImageCopy(status_image,logo_image,x1,y,0,0,logo_image->sx,logo_image->sy);
					gdImageDestroy(logo_image);
				        }

				/* if not... */
				else{

					/* get upper left hand corner of bounding box */
					x1=x-(node_width/2);
					x2=x1+node_width;
					y2=y+node_height;

					/* last ditch effort - draw a host bounding box */
					draw_line(x1,y,x2,y,color);
					draw_line(x1,y,x1,y2,color);
					draw_line(x1,y2,x2,y2,color);
					draw_line(x2,y2,x2,y,color);
				        }
		                }
		        }
	        }

	/* display a pretty icon and some status information for the host... */
	else{

		/* see if user is authorized to view this host - if they're not, we'll just display an unknown icon... */
		authorized_for_host=is_authorized_for_host(hst,&current_authdata);

		/* get corners of host bounding box */
		x1=x-(node_width/2);
		x2=x1+node_width;
		y2=y+node_height;


		if(create_type==CREATE_IMAGE){

			/* get the center of the host bounding box */
			centerx=((x2-x1)/2)+x1;
			centery=((y2-y)/2)+y;

			/* see if this host has extended data associated with it (i.e. an icon) */
			temp_hostextinfo=find_hostextinfo(hst->name);
			if(temp_hostextinfo!=NULL && strcmp(temp_hostextinfo->gd2_icon_image,""))
				has_image=TRUE;

			/* if the user isn't authorized to view this host, just place the unknown image here... */
			if(authorized_for_host==FALSE)
				has_image=FALSE;

			/* if the host doesn't have an image associated with it (or the user doesn't have rights to see this host), use the unknown image */
			if(has_image==FALSE){
				if(unknown_logo_image!=NULL)
					gdImageCopy(status_image,unknown_logo_image,centerx-(unknown_logo_image->sx/2),y,0,0,unknown_logo_image->sx,unknown_logo_image->sy);
		                }

			/* else load the logo associated with this host */
			else{

				/* get the name of the image file to open for the logo */
				snprintf(image_input_file,sizeof(image_input_file),"%s%s",physical_logo_images_path,temp_hostextinfo->gd2_icon_image);
				image_input_file[sizeof(image_input_file)-1]='\x0';

				/* read in the image... */
				logo_file=fopen(image_input_file,"r");
				if(logo_file!=NULL){

					logo_image=gdImageCreateFromGd2(logo_file);
					fclose(logo_file);

				        /* copy the logo to the image being generated... */
					if(logo_image!=NULL){
						gdImageCopy(status_image,logo_image,centerx-(logo_image->sx/2),y,0,0,logo_image->sx,logo_image->sy);
						gdImageDestroy(logo_image);
			                        }
			                }

				/* try to use the unknown icon if we couldn't load the image... */
				else if(unknown_logo_image!=NULL)
					gdImageCopy(status_image,unknown_logo_image,centerx-(unknown_logo_image->sx/2),y,0,0,unknown_logo_image->sx,unknown_logo_image->sy);
				else{

					/* last ditch effort - draw a host bounding box */
					draw_line(x1,y,x2,y,color);
					draw_line(x1,y,x1,y2,color);
					draw_line(x1,y2,x2,y2,color);
					draw_line(x2,y2,x2,y,color);
				        }
		                }

			/* draw the name of the host using the small font... */
			snprintf(temp_buffer,sizeof(temp_buffer),"%s",(authorized_for_host==FALSE)?"UNKNOWN":hst->name);
			temp_buffer[sizeof(temp_buffer)-1]='\x0';
			string_height=gdFontSmall->h;
			string_width=gdFontSmall->w*strlen(temp_buffer);
			gdImageString(status_image,gdFontSmall,centerx-(string_width/2),y2-(string_height*2)-2,(unsigned char *)temp_buffer,color_black);
		        }


		/* if the user isn't authorized to view information on this host, don't print any more information... */
		if(authorized_for_host==FALSE)
			return;


		if(create_type==CREATE_IMAGE){

			/* find the status entry for this host */
			temp_status=find_hoststatus(hst->name);

			/* get the status of the host (pending, up, down, or unreachable) */
			if(temp_status!=NULL){

				/* draw the status string */
				if(temp_status->status==HOST_DOWN){
					strncpy(temp_buffer,"Down",sizeof(temp_buffer));
					status_color=color_red;
		                        }
				else if(temp_status->status==HOST_UNREACHABLE){
					strncpy(temp_buffer,"Unreachable",sizeof(temp_buffer));
					status_color=color_red;
		                        }
				else if(temp_status->status==HOST_UP){
					strncpy(temp_buffer,"Up",sizeof(temp_buffer));
					status_color=color_green;
		                        }
				else if(temp_status->status==HOST_PENDING){
					strncpy(temp_buffer,"Pending",sizeof(temp_buffer));
					status_color=color_grey;
		                        }
				else{
					strncpy(temp_buffer,"Unknown",sizeof(temp_buffer));
					status_color=color_orange;
			                }

				temp_buffer[sizeof(temp_buffer)-1]='\x0';

				/* write the host status string to the generated imate... */
				string_width=gdFontSmall->w*strlen(temp_buffer);
				gdImageString(status_image,gdFontSmall,centerx-(string_width/2),y2-string_height-2,(unsigned char *)temp_buffer,status_color);
		                }
		        }

		if(create_type==CREATE_HTML){

			/* start the imagemap area... */
			printf("<AREA shape='rect'");

			/* write out coordinates for the image are associated with this host (scale if necessary) */
			if(scale_image==TRUE)
				printf(" coords='%d,%d,%d,%d'",(int)((double)x1*scaling_factor),(int)((double)y*scaling_factor),(int)((double)x2*scaling_factor),(int)((double)y2*scaling_factor));
			else
				printf(" coords='%d,%d,%d,%d'",x1,y,x2,y2);

			printf(" href='%s?host=%s",STATUSMAP_CGI,hst->name);

			/* print CGI options... */
			if(use_node_width==TRUE)
				printf("&nodewidth=%d",node_width);
			if(use_node_height==TRUE)
				printf("&nodeheight=%d",node_height);
			if(use_horizontal_spacing==TRUE)
				printf("&hspacing=%d",horizontal_spacing);
			if(use_vertical_spacing==TRUE)
				printf("&vspacing=%d",vertical_spacing);
			if(use_max_width==TRUE)
				printf("&maxwidth=%d",max_image_width);
			if(use_max_height==TRUE)
				printf("&maxheight=%d",max_image_height);

			/* end of CGI options and HREF... */
			printf("'");

			/* begin printing the ALT tag for this image area... */
			printf(" alt='%s",hst->alias);
			if(temp_hostextinfo!=NULL)
				printf(" (%s)",temp_hostextinfo->icon_image_alt);
			printf(":");

			/* get service status information for use in ALT tag of image area... */
			service_totals=get_service_count(hst,SERVICE_OK)+get_service_count(hst,SERVICE_RECOVERY);
			if(service_totals>0){
				found_total=1;
				printf("%d ok",service_totals);
		                }
			service_totals=get_service_count(hst,SERVICE_CRITICAL)+get_service_count(hst,SERVICE_UNREACHABLE)+get_service_count(hst,SERVICE_HOST_DOWN);
			if(service_totals>0){
				printf("%s%d critical",(found_total==1)?", ":"",service_totals);
				found_total=1;
		                }
			service_totals=get_service_count(hst,SERVICE_WARNING);
			if(service_totals>0){
				printf("%s%d warning",(found_total==1)?", ":"",service_totals);
				found_total=1;
		                }
			service_totals=get_service_count(hst,SERVICE_UNKNOWN);
			if(service_totals>0){
				printf("%s%d unknown",(found_total==1)?", ":"",service_totals);
				found_total=1;
		                }
			service_totals=get_service_count(hst,SERVICE_PENDING);
			if(service_totals>0)
				printf("%s%d pending",(found_total==1)?", ":"",service_totals);

			/* the end of the ALT tag for the image area... */
			printf("'>\n");
	                }
	        }

	return;
	}




/* draws a link to a child host */
void draw_child_link(host *hst,int x0,int y0,int x1,int y1,int color){

	if(create_type==CREATE_HTML)
		return;

	/* draw the link */
	draw_line(x0,y0,x1,y1,color);	

	return;
	}



/* draws a line */
void draw_line(int x1,int y1,int x2,int y2,int color){

	if(create_type==CREATE_HTML)
		return;

	/* draws a line */
	gdImageLine(status_image,x1,y1,x2,y2,color);

	return;
	}

